// DIB_CONFIG.JS
//
// Author:	Maud-Dib
// Version:	2.06
//
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
//
// Description:
//
// Contains the default values for configuration variables not contained in the decal panels.
//
// DO NOT CHANGE ANYTHING IN THIS FILE!!!
//
// If you want to set a different value for one or more of these variables, copy the variable
// definitions to your userconfig.dat file in C:\DIBBOTCFG.  If you place all your
// changes in userconfig.dat then it will be easier for you to see what variables you have
// modified, and your changes will not be wiped out by the next release of the program. 


// ***************************************************************************************************
//

// Macro Control Flags.  Not settable from decal panels
flagCheckVitals = true;		// Do we want to check vitals? Might disable during testing

// Flags that control logging of information
logChat		= true;		// If true, log tells/local chat to the event log
logIdle		= true;		// If true, log information related to IDLE state processing
logVitals	= true;		// If true, log when stam/mana/health related spells cast
logExpire	= true;		// If true, log when spells on us expire
logDanger	= true;		// If true, log when Danger flag changes state
logPlayers	= true;		// If true, log players detected in area to event log
logComp		= true;		// If true, log component shortages to event file
logGifts	= true;		// If true, log stuff people give to us to event log
logBuffs	= true;		// If true, log all buff casts to event file.
logBurden	= true;		// If true, log info related to burden
logCmds		= true;		// If true, log info related to command processing
logDecal	= true;		// If true, log info related to decal control processing
logPlig		= true;		// If true, log when players in game state (skapi.plig) changes
logSummon	= true;
logReset	= true;

// Debug flags.  Macro operation can be severely slowed down when using these flags, so be careful.
debug2		= false;	// Will log extended debugging information for non-events
debug3		= false;	// Special debugging information for non-events
debug4		= false;	// Special debugging information for non-events
debug5		= false;	// Special debugging information for non-events
debugAssCre	= false;	// Log debug information for assessing creatures
debugEvent2	= false;	// Will log extended debugging information for events
debugEvent3	= false;	// Special debugging information for events
debugEvent4	= false;	// Special debugging information for events
debugInit	= false;	// Show debug information related to INIT state processing
debugPrev	= false;	// Show debug info related to previous yields/vulns on creatures
debugEvade	= false;	// Show debugging information related to evading war spells
debugInvent 	= false;	// If true, show information related to inventory management (selling, giving)
debugAddInv	= false;	// If true, log all items added to inventory
debugVitals	= false;	// Show debugging information for vitals management (mana, stam, health)
debugDecal	= false;	// If true, show debugging information related to decal control processing
debugCmds	= false;	// Show debugging information for command processing
debugComps  	= false;	// Show debugging information for component management
debugEval	= false;	// Show debugging information for the evaluate command
debugDanger	= false;	// Show info related to danger and critical condition detection
debugLog	= false;	// Show debugging info related to player logon/logoff
debugReset	= false;


// *******************************************************************************************************
//
// Main settings

accDefChar	= 3;		// Default char to log when macro started at login screen.  Chars numbered 0 to 6.		
portalMinInt	= 30*1000;	// Minimum interval (ms) between portal casts of same portal. Prevents
				// malicious player from summoning same portal over and over.
				


// These players on the shitlist.  Will ignore all messages from these players
ignoreList 	= new Array("Kou","Jhesus","May-hem");

// Don't record chat for these chars (these are bots in vicinity)
chatIgnoreList	= new Array();

// Message to broadcast after successful portal open.
broadOpenMsg	= "Donations appreciated.";

// Message to broadcast periodically if enabled
broadIntAdver	= 4*60*1000;	// Minimum msecs between advertisement requests (def 4 minutes)
broadAdverMsg	= "Welcome to DibBot " + macroVersion +
		  ".  You can now summon by using portal name in local chat";

accLogonWait	= 5000;			// Amount of time to wait to login a character after a logout.  If
					// we try to login too fast the mouse movement commands don't work.

playerLogoutTimeout	= 35*1000;	// How long to wait (ms) before assuming Logout failed
playerLoginTimeout	= 35*1000;	// How long to wait (ms) before assuming Login failed
playerLogoutAttempts	= 3;		// How many times to attempt logout if we fail
playerLoginAttempts	= 3;		// How many times to attempt login if we fail

statDumpInt		= 60*60*1000;

closebyDist		= 0.30;

fellowName		= "Portal Bot"	// Fellowship name if creFellow set to 'true'
fellowInt		= 15*1000;	// Wait this many milliseconds after logging on character to
					// create fellow.  Enables us to summon faster and if there
					// are multiple requests we don't waste time creating fellow

// *******************************************************************************************************
//
// Miscellaneous
buffMaxFiles	= 10;		// Maximum # of BUFFCONFIGxx.DAT files

charModeDelay   = 600;		// Time to delay after switching from peace to war mode
logoutTimeout	= 30000;	// # of milliseconds to wait after initiating player logoff for player to
				// go to login screen.  Give up after this value (default 30 seconds)
buffMaxInt	= 2*60*1000;	// Maximum number of time (milliseconds) we can stay in buff state continuously.
inactTimer	= 0;		// If set, changes the inactivity logoff timer to this (seconds)
autoRotateTime	= 250;		// How long to press left key before casting portal.  Used to rotate character
				// so it doesn't cast a portal on top of previous casts.
keyPressInt	= 10*60*1000;	// Press a key every this many milliseconds to prevent inactivity logout.
busyWait	= 4000;		// Amount of time to wait when pointer is busy before initiating reset attempt
locRecordThresh	= 0.20;		// Record location changes more than this many map units
autoResumeInt	= 11*60*1000;	// Time (ms) after pausing macro to resume macro (if enabled)

// ********************************************************************************************************
//
// Component management parameters
//
// Note actual components to check defined in CompInit() function below
compInterval	= 60*1000;	// How often to check comps (ms).  Default is 60 seconds
compCheckMax	= 3*60*1000;	// Max amount of time (ms) we can stay in the CompCheck state, def 3 minutes
compSpamInt	= 3*60*1000;	// How often to sound an alert for a comp that is low

// ********************************************************************************************************
//
// Command parameters

enableCommands	= true;		// Enable processing of commands from people on the authList defined below
authAllegiance	= false;	// Allow anybody in allegiance to use admin commands

// These players can give admin commands to us.
authList	= new Array();

// whereto holddown
wheretoHolddown = 5000;		// Don't respond to whereto in @cg within this many seconds of last request

// ********************************************************************************************************
//
// Health, Mana, and Stamina Management
//
// The following set of variable control when we heal ourselves
threshHealOffset    = 70;	// When health this much below current maximum, then heal self. Note
				// that if we are attacking a creature a drainhealth is a possibility
				// so this should not be set lower than 'threshDrainHealthOffset'.


// The next set of variable control when we cast stam to mana
threshManaOffset    = 150;	// When mana this much below maximum, then do stam-to-mana (unless stam low)
threshStaminaMin    = 120;	// If stamina is below this level (absolute) then don't do stam-to-manas
				// since the mana received at this low stam level won't be much more than
				// cost of spell.

// These variable control when a self revitalize occurs
threshStaminaOffset =  90;	// When stamina this much below maximum, then cast revitalize self.

// Miscellaneous control variables related to stats
threshBuffMana	    = 80;	// Only try to buff if mana at least this level


// ********************************************************************************************************
//
// War Avoidance Info
avoidDangerThresh = 0.005;	// If war spell is calculated to come this close to us, then take
				// steps to avoid.

avoidMinDist	= 0.01;		// Minimum distance away from us war bolt must be detected before we
				// attempt to move out of the way.  Any closer than 0.05 and we will
				// have trouble moving out of the way quick enough.  Do not set to really
				// low value or we will attempt to avoid our own war spells!

avoidTime	= 600;		// How much time we should sidestep in milliseconds.  600 means move
				// for 0.6 seconds

avoidTimeSlide	= 650;		// How much time we should slide cast

avoidSlideHold	= 400;		// Don't attempt 2 slide casts within this many milliseconds of each other.
				// Needed because the slidecast routine fires every time a war spell comes at
				// us and if several come at the same time we would slide cast more then needed.

avoidHoldDown	= 2000;		// Number of millseconds before evading back in the opposite direction
				// Needed when two war spells come in close succession that we don't
				// end up in the original location where the 1st spell will hit us.

avoidMaxAngle	= 45;		// If the difference between the war spell's heading to its target and
				// to us is more than this, then there is no way it can hit us so abort
				// the processing of this spell.  We need this to prevent false positives
				// when the war spell is heading ~180 degrees away from us.

// ********************************************************************************************************
//
// Alert and support channel settings

supportChars	= new Array();
				// Any characters in this list will receive @tells when important events happen
				// in the macro, for example low comps, creature detected nearby, and deaths.
				// Example: supportChars = new Array("Boy Gary","Crackhead Bob");

alertAbortCount = 5;		// # of times to play abort alert
alertAbortInt	= 1000;		// milliseconds between abort alerts

alertDeathWav	= skapi.WavNew(macroDir + "dead.wav");		// Sound to play when we die
alertCompsWav	= skapi.WavNew(macroDir + "portal.wav");	// Played when comps low
alertAbortWav	= skapi.WavNew(macroDir + "vassel.wav");	// Played when macro unexpectedly stops
alertHideWav	= skapi.WavNew(macroDir + "peekay.wav");	// Played when we hide (autolog)

// Interval to hide (ms) when creature detected. 
creWarnDist	= 0.20;
creDangerDist	= 0.10;
creWarnInt	= 3*60*1000;		// Default spam interval is 3 minutes
creListPlayers	= true;			// Dump players nearby anytime warning is issued

hideInt		= 6*60*1000;		// Default hide interval is 6 minutes.
creWarnMsg	= "Creature detected nearby, we will log if it gets closer!";
creDangerMsg	= "Creature detected closeby, hiding...!";