// DIB_CONSTANTS.JS
//
// Author:	Maud-Dib
// Version:	2.06
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
//
// Description:
//
//	Defines constants for DibBot
//
// Modification History:
//
// Version Date		Who	Description
// 1.01	   09/01/02	Maid-Dib	Initial version
//
//

// **************************************************************************************************
// Global Macro Context
//

// Version information
macroVersion = "V2.08";

// *******************************************************************************************************
//
// Locations for log and config files.  Note \\ is required to create one \ character in the strings.
// Also note the macroDir constants already has the backslashes in the name, while configDir does not.


// Directory of macro files.  Needed so we can locate configuration files.  Note \\ translates to
// a single \ in Jscript code
macroDir  = "C:\\DibBot208\\";
configDir = "C:\\DibBotCfg" 				// Don't add the \\ at the end of this one
accountDir = configDir + "\\" + skapi.szAccount;

fileBuffs	= macroDir + "buffs.csv";
fileRunLog	= macroDir + "diblog_run.txt";
fileGiftLog 	= macroDir + "diblog_gifts.txt";
fileStatLog 	= macroDir + "diblog_stats.txt";
fileChatLog 	= macroDir + "diblog_chat.txt";
fileCmdLog	= macroDir + "diblog_command.txt";

fileGenCfg	= accountDir + "\\genconfig.dat";	// General config file.  Same file for all characters

fileUserCfg	= accountDir + "\\userconfig.dat";	// User config file. Same for all chars

fileBuffCfgPre  = accountDir + "\\buffconfig_";		// Buff config files have character name or number
fileBuffCfgPost = ".dat";				// embedded in name so break down into pre and post parts.

fileCompCfgPre  = accountDir + "\\compconfig_";		// Comp config files have character name or number
fileCompCfgPost = ".dat";				// embedded in name so break down into pre and post parts.

// Set current version of config files.  Version information placed at the beginning of every config file
// in case we need to change the structure in the future.
verGenCfg	= 1;		// Current version of general configuration file
verBuffCfg	= 1;		// Current version of buff configuration file

// Possible macro states
Count = 1;
stateInit		= Count++;
stateIdleInit		= Count++;
stateIdle		= Count++;
stateSummonInit		= Count++;
stateSummon		= Count++;
stateCheckComps		= Count++;
stateHide		= Count++;
stateReset		= Count++;
stateStormed		= Count++;
stateDied		= Count++;
stateLog		= Count++;
stateExit		= Count++;

textState = new Array();
textState[stateInit] 		= "Init";
textState[stateIdleInit] 	= "IdleInit";
textState[stateIdle] 		= "Idle";
textState[stateSummonInit] 	= "SummonInit";
textState[stateSummon] 		= "Summon";
textState[stateCheckComps] 	= "CheckComps";
textState[stateHide] 		= "Hide";
textState[stateReset] 		= "Reset";
textState[stateStormed] 	= "Stormed";
textState[stateDied] 		= "Died";
textState[stateLog] 		= "Log";
textState[stateExit] 		= "Exit";

// Macro substates
substateNone			= 0;

// Summon substates
stateSummonPortal		= 1;	// Summons portal
stateSummonGem			= 2;	// Uses portal gem
stateSummonDone			= 3;	// Cleanup


// **************************************************************************************************
//
// Summon Context Constants

// Portal spell ids for summon levels
summonPrimary   = new Array( 0,  157,  158, 1637 );
summonSecondary = new Array( 0, 2648, 2649, 2650 );

maxSlots = 10;

portalGemPost	= " Portal Gem";
portalGems	= new Array( "Al-Arqas", "Holtburg", "Lytelthorpe", "Nanto", "Rithwic",
			"Samsur", "Shoushi", "Yanshi", "Yaraq" );
portalGemList	= portalGems.join(", ");

// **************************************************************************************************
//
// Idle Context Constants

// Constants
idleMin		= 10*60*1000;	// Force Idle state after this many milliseconds


// **************************************************************************************************
//
// Spell Context Constants
spellMaxLvl	= 7;	// Maximum spell level possible

// Spell ids for critical spells (all levels)
spellListStamMana	= new Array(0, 1676, 1677, 1678, 1679, 1680, 1681, 2345);
spellListHeal		= new Array(0,    6, 1157, 1158, 1159, 1160, 1161, 2073);
spellListRevitalize	= new Array(0, 1177, 1178, 1179, 1180, 1181, 1182, 2083);
spellListStrength	= new Array(0,    2, 1328, 1329, 1330, 1331, 1332, 2087);


// **************************************************************************************************
//
// Miscellaneous Constants
NoCheck = true;		// Define symbolic name for true boolean value
ToChat	= true;		// Define another symbolic name for true boolean value