// DIB_CONTROLEVENT.JS
//
//
// Author:	Maud-Dib
// Version:	2.06
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
//
// Description:
//
//	The following file contains all the event function OnControlEvent which
//	handles all the Decal control panel generated events
//

// Handler function for the OnControl event.
function MainHandlers_OnControlEvent(szPanel, szControl, dictSzValue) {
    if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: Begin, Panel [" + szPanel + "] Control [" + szControl + "]");

    if (szPanel == szPanelControl) {
   	// Process Control Panel Controls
	switch (szControl) {

	    // C1 tab **********************************************

	    case "btnPause":
		szCurCap = dictSzValue("btnPause");
		if (szCurCap == "Pause") {
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Pause request detected.  Pausing macro...");
		    Chat.Screen("Pausing macro...");
		    MacroPaused = true;
		    skapi.SetControlProperty(szPanelControl, "btnPause", "Text", "Resume");
		} else {
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Resume request detected.  Resuming macro...");
		    Chat.Screen("Resuming macro...");
		    MacroPaused = false;
		    skapi.SetControlProperty(szPanelControl, "btnPause", "Text", "Pause");
		} // endif
 	        break;

	    case "btnSummonMode":
		szCurCap = dictSzValue("btnSummonMode");
		if (szCurCap == "Disable Summoning") {
		    Chat.Screen("Summoning portals disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Summoning portals disabled.");
		    SummonMode = false;
		    skapi.SetControlProperty(szPanelControl, "btnSummonMode", "Text", "Enable Summoning");
		} else {
		    Chat.Screen("Summoning portals enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Summoning portals enabled.");
		    SummonMode = true;
		    skapi.SetControlProperty(szPanelControl, "btnSummonMode", "Text", "Disable Summoning");
		} // endif
 	        break;

	    case "btnBuffMode":
		szCurCap = dictSzValue("btnBuffMode");
		if (szCurCap == "Disable Buffing") {
		    Chat.Screen("WARNING!!! Buffing disabled!", cmcYellow);
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Buffing disabled");
		    BuffMode = false;
		    skapi.SetControlProperty(szPanelControl, "btnBuffMode", "Text", "Enable Buffing");
		} // endif

		if (szCurCap == "Enable Buffing") {
		    Chat.Screen("Buffing enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Buffing enabled");
		    BuffMode = true;
		    skapi.SetControlProperty(szPanelControl, "btnBuffMode", "Text", "Disable Buffing");
		} // endif
 	        break;

	    case "btnPlayers":
	    	var OurLoc = skapi.maplocCur;
		var PlayerAcf = skapi.AcfNew();
    		PlayerAcf.ocm = ocmPlayer;
		var PlayerList = PlayerAcf.CoacoGet();
		var PlayCnt = 0;

		for (var index=0; index<PlayerList.Count; index++) {
		    var PlayerAco = PlayerList.Item(index);
		    // If this Aco is us, then continue with next entry
		    if (PlayerAco.szName == skapi.szName) continue;

		    // Get player distance.  Exclude players too far from us
		    // which most likely means a stale entry in the object table.
		    var Dist = Util.Distance(PlayerAco.maploc,OurLoc);
		    if (Dist > 3.0) continue;

		    // Report player distance and heading
		    PlayCnt++;
		    var OutLine = "Name: " + Util.Width(PlayerAco.szName,2) + "  Dist: " + Util.RoundFill(Dist,2);
		    if (Dist > 0.01)
			OutLine = OutLine + "  Head: " + Util.Round(OurLoc.HeadToMaploc(PlayerAco.maploc),0) +
					    "  Coords:  " + Util.FormatCoords(PlayerAco.maploc,2);
	    	    Chat.Screen(OutLine);
		} // endfor

		if (PlayCnt == 0) {
		    Chat.Screen("No players near us.");
		    break;
		} // endif

		if (PlayCnt == 1)
		    Chat.Screen ("1 player detected.");
		else
		    Chat.Screen(PlayCnt + " players detected.");
		break;

	    case "btnStats":
	    	BuildGenStats();
	    	BuildPortalStats();
		Chat.Screen("ElapsedTimeMin: " + ElapsedTimeMin);
		Chat.Screen(" ");
		for (var x=0; x<StatList.length; x++) Chat.Screen(StatList[x]);
		Chat.Screen(" ");
	        break;

	    case "btnShowComps":
	    	ReportComps();
	        break;

	    case "btnVersion":
	    	Chat.Screen("Running PortalBot " + macroVersion);
	        break;

	    case "btnResetStats":
	    	Chat.Screen("Resetting stat counters...");
	    	if (logDecal||debugDecal) Log.OutD("OnControlEvent: Resetting stat counters...");
	    	ResetStats();
	        break;

	    case "btnForceAll":
	    	if (logDecal||debugDecal) Log.OutD("OnControlEvent: Setting force flag on all spells.");
		var ForceCount = Buff[AccCurChar].ForceAll();
		Chat.Screen("Forced " + ForceCount + " spells.");
		if (logDecal||debugDecal) Log.OutD("Forced " + ForceCount + " spells.");
 	        break;


	    // C2 tab **********************************************


	    case "btnChar0":
	        if (AccCurChar == 0) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 0;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar1":
	        if (AccCurChar == 1) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 1;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar2":
	        if (AccCurChar == 2) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 2;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar3":
	        if (AccCurChar == 3) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 3;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar4":
	        if (AccCurChar == 4) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 4;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar5":
	        if (AccCurChar == 5) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 5;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;
		
	    case "btnChar6":
	        if (AccCurChar == 6) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 6;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar7":
	        if (AccCurChar == 7) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 7;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar8":
	        if (AccCurChar == 8) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 8;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;

	    case "btnChar9":
	        if (AccCurChar == 9) {
		    Chat.Screen("You are already on that character!");
		    break;
		} // endif
		SwitchChar = 9;
		Chat.Screen("Switching to character " + skapi.rgszChar(SwitchChar) + " next idle period.");
		break;


	    // Shortcut tab ******************************************************

	    case "btnFillComps":
	        Chat.Say("@fillcomps");
	        break;

	    case "btnLifeStone":
		Chat.Say("@lifestone");
	        break;

	    case "btnHouse":
	    	Chat.Say("@house recall");
		break;

	    case "btnMansion":
		Chat.Say("@house mansion_recall");
		break;

	    case "btnMarket":
		Chat.Say("@marketplace");
		break;

	    case "btnStoneRecall":
	        SpellObj_LifestoneRecall = Cast.GetSpellObj("Lifestone Recall");
	        if (!Cast.Know(SpellObj_LifestoneRecall.spellid)) {
	            Chat.Screen("You do not know the recall to lifestone spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Lifestone Recall spell...");
	        Cast.Cast(SpellObj_LifestoneRecall);
		break;

	    case "btnPortal1Recall":
	        SpellObj_Portal1Recall = Cast.GetSpellObj("Primary Portal Recall");
	        if (!Cast.Know(SpellObj_Portal1Recall.spellid)) {
	            Chat.Screen("You do not know the Primary Portal Recall spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Primary Portal Recall spell...");
	        Cast.Cast(SpellObj_Portal1Recall);
	        break;

	    case "btnPortal2Recall":
	        SpellObj_Portal2Recall = Cast.GetSpellObj("Secondary Portal Recall");
	        if (!Cast.Know(SpellObj_Portal2Recall.spellid)) {
	            Chat.Screen("You do not know the Secondary Portal Recall spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Secondary Portal Recall spell...");
	        Cast.Cast(SpellObj_Portal2Recall);
	        break;

	    case "btnPortalRecall":
	        SpellObj_PortalRecall = Cast.GetSpellObj("Portal Recall");
	        if (!Cast.Know(SpellObj_PortalRecall.spellid)) {
	            Chat.Screen("You do not know the Portal Recall spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Portal Recall spell...");
	        Cast.Cast(SpellObj_PortalRecall);
	        break;

	    case "btnRecallSanct":
	        SpellObj_RecallSanct = Cast.GetSpellObj("Recall the Sanctuary");
	        if (!Cast.Know(SpellObj_RecallSanct.spellid)) {
	            Chat.Screen("You do not know the recall to sanctuary spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Recall the Sanctuary spell...");
	        Cast.Cast(SpellObj_RecallSanct);
	        break;

	    case "btnUlgrimRecall":
	        SpellObj_UlgrimRecall = Cast.GetSpellObj("Ulgrim's Recall");
	        if (!Cast.Know(SpellObj_UlgrimRecall.spellid)) {
	            Chat.Screen("You do not know the Ulgrim's Recall spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Ulgrim's Recall spell...");
	        Cast.Cast(SpellObj_UlgrimRecall);
	        break;

	    case "btnCaulRecall":
	        SpellObj_CaulRecall = Cast.GetSpellObj("Recall to the Singularity Caul");
	        if (!Cast.Know(SpellObj_CaulRecall.spellid)) {
	            Chat.Screen("You do not know the Recall to the Singularity Caul spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Recall to the Singularity Caul spell...");
	        Cast.Cast(SpellObj_CaulRecall);
	        break;

	    case "btnAphusRecall":
	        SpellObj_AphusRecall = Cast.GetSpellObj("Recall Aphus Lassel");
	        if (!Cast.Know(SpellObj_AphusRecall.spellid)) {
	            Chat.Screen("You do not know the Recall Aphus Lassel spell!");
	            break;
	        } // endif
	        Chat.Screen("Casting Recall Aphus Lassel spell...");
	        Cast.Cast(SpellObj_AphusRecall);
	        break;


	    // D(ebug) tab  *********************************************

	    case "btnShowBuffs":
		Chat.Screen("Displaying buff structures...");
	    	Buff[AccCurChar].ShowBuffs();
	    	break;

	    case "btnAnalSel":
	        var Aco = skapi.acoSelected;
		if ((Aco == undefined) || (Aco == null)) {
		    Chat.Screen("No object selected.");
		    break;
		} // endif

		var wid = 1.5;
	        Chat.Screen("");
	        Chat.Screen(Util.Width("Oid",wid) 		+ ": " + Util.Hex(Aco.oid));
	        Chat.Screen(Util.Width("fExists",wid) 		+ ": " + Aco.fExists);
	        Chat.Screen(Util.Width("szName",wid)		+ ": " + Aco.szName);
	        Chat.Screen(Util.Width("szPlural",wid)		+ ": " + Aco.szPlural);
	        Chat.Screen(Util.Width("oty",wid) 		+ ": " + Util.Hex(Aco.oty));
	        Chat.Screen(Util.Width("ocm",wid) 		+ ": " + Util.Hex(Aco.ocm));
	        Chat.Screen(Util.Width("olc",wid) 		+ ": " + Util.Hex(Aco.olc));
	        Chat.Screen(Util.Width("eqm",wid) 		+ ": " + Util.Hex(Aco.eqm));
	        Chat.Screen(Util.Width("eqmWearer",wid) 	+ ": " + Util.Hex(Aco.eqmWearer));
	        Chat.Screen(Util.Width("mcm",wid) 		+ ": " + Util.Hex(Aco.mcm));
	        Chat.Screen(Util.Width("szMcm",wid) 		+ ": " + Aco.szMcm);
	        Chat.Screen(Util.Width("cpyValue",wid) 		+ ": " + Aco.cpyValue);
	        Chat.Screen(Util.Width("material",wid)		+ ": " + Aco.material);
	        Chat.Screen(Util.Width("workmanship",wid)	+ ": " + Aco.workmanship);
	        Chat.Screen(Util.Width("citemStack",wid) 	+ ": " + Aco.citemStack);
	        Chat.Screen(Util.Width("citemMaxStack",wid) 	+ ": " + Aco.citemMaxStack);
	        Chat.Screen(Util.Width("cuseLeft",wid) 		+ ": " + Aco.cuseLeft);
	        Chat.Screen(Util.Width("cuseMax",wid) 		+ ": " + Aco.cuseMax);
	        Chat.Screen(Util.Width("burden",wid) 		+ ": " + Aco.burden);
	        Chat.Screen(Util.Width("fInInventory",wid) 	+ ": " + Aco.fInInventory);
	        Chat.Screen(Util.Width("iitem",wid) 		+ ": " + Aco.iitem);
	        Chat.Screen(Util.Width("spellid",wid) 		+ ": " + Aco.spellid);
	        Chat.Screen(Util.Width("distApproach",wid) 	+ ": " + Aco.distApproach);
	        Chat.Screen(Util.Width("fContainer",wid) 	+ ": " + Aco.fContainer);
	        Chat.Screen(Util.Width("citemMax",wid) 		+ ": " + Aco.citemMax);
	        Chat.Screen(Util.Width("citemContents",wid) 	+ ": " + Aco.citemContents);
	        Chat.Screen(Util.Width("icon",wid) 		+ ": " + Aco.icon);
	        Chat.Screen(Util.Width("model",wid) 		+ ": " + Aco.model);
	        if ((Aco.maploc != undefined) && (Aco.maploc != null)) {
		    // Map loc seems to exist, print it out and the distance to us and our current loc
		    Chat.Screen(Util.Width("coords",wid) 	+ ": " + Util.FormatCoords(Aco.maploc,2));
		    Chat.Screen(Util.Width("Cur loc dist",wid)  + ": " +
			Util.RoundFill(Util.Distance(Aco.maploc,skapi.maplocCur),2));
		} // endif
	        break;

	    case "editCmd":
		szCmd = (dictSzValue("editCmd"));
		Chat.Screen("Evaluating command [" + szCmd + "]");
		eval(szCmd);
		break;

 	} // end switch
 	if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: End, Control");
    	return;
    } // endif

    // Config panel
    if (szPanel == szPanelConfig) {
   	// Process Control Panel Controls
	switch (szControl) {

	    // G(eneral)1 tab ******************************************

	    case "btnGenLoad":
	       	Chat.Screen("Loading general configuration info...");
	        var Status = GenLoad();	// Load info from file
	        if (Status) {
	            CritSpellsInit();	// Redefine spell objects
	            SpellCheck(true);	// Make sure critical spells known
		    SetGenControls();	// Now update decal panels with new settings
		    Chat.Screen("Loaded general configuration info from file " + GenFileName);
		} else {
		    Chat.Screen("File " + GenFileName + " not found!");
		} // endif
	        break;

	    case "btnGenSave":
	       	Chat.Screen("Saving general configuration info...");
		GenSave();
		Chat.Screen("General configuration info saved to file " + GenFileName);
	        break;

	    case "btnGenReset":
	    	Chat.Screen("Resetting...");
		GenVarInit();		// Load default values
		CritSpellsInit();	// Redefine spell objects
		SpellCheck(true);	// Make sure critical spells known
		SetGenControls();	// Now update decal panels with new settings
		Chat.Screen("General configuration reset to defaults!");
	        break;

	    case "chkCreatureHide":
	    	flagCreatureLog = !flagCreatureLog;
		skapi.SetControlProperty(szPanelConfig, "chkCreatureHide", "checked", flagCreatureLog );
		if (flagCreatureLog) {
		    if (flagVerbose) Chat.Screen("Logoff on creature detect enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on creature detect enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("WARNING!!! Logoff on creature detect disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on creature detect disabled.");
		} // endif
 	        break;

	    case "chkCreatureWarn":
	    	flagCreatureWarn = !flagCreatureWarn;
		skapi.SetControlProperty(szPanelConfig, "chkCreatureWarn", "checked", flagCreatureWarn );
		if (flagCreatureWarn) {
		    if (flagVerbose) Chat.Screen("Warn on creature detect enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Warn on creature detect enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Warn on creature detect disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Warn on creature detect disabled.");
		} // endif
 	        break;

	    case "chkDeathLog":
	    	flagDeathLog = !flagDeathLog;
		skapi.SetControlProperty(szPanelConfig, "chkDeathLog", "checked", flagDeathLog );
		if (flagDeathLog) {
		    if (flagVerbose) Chat.Screen("Logoff on death enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on death enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("WARNING!!! Logoff on death disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on death disabled.");
		} // endif
 	        break;

	    case "chkDamageLog":
	    	flagDamageLog = !flagDamageLog;
		skapi.SetControlProperty(szPanelConfig, "chkDamageLog", "checked", flagDamageLog );
		if (flagDamageLog) {
		    if (flagVerbose) Chat.Screen("Logoff on damage enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on damage enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("WARNING!!! Logoff on damage disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff on damage disabled.");
		} // endif
 	        break;

	    case "chkReduceBurden":
		flagReduceBurden = !flagReduceBurden;
		skapi.SetControlProperty(szPanelConfig, "chkReduceBurden", "checked", flagReduceBurden );
		if (flagReduceBurden) {
		    if (!Cast.Know(SpellObj_Strength.spellid)) {
			Chat.Screen("Spell Strength Self not known, disabling option.", ToChat, cmcYellow);
			flagReduceBurden = false;
			skapi.SetControlProperty(szPanelConfig, "chkReduceBurden", "checked", false );
		    } // endif
		    if (flagVerbose) Chat.Screen("Burden reduction enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Burden reduction enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Burden reduction disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Burden reduction disabled.");
		} // endif
		break;

	    case "chkAutoResume":
	    	flagAutoResume = !flagAutoResume;
		skapi.SetControlProperty(szPanelConfig, "chkAutoResume", "checked", flagAutoResume);
		if (flagAutoResume) {
		    if (flagVerbose) Chat.Screen("Macro will automatically resume from Paused state after " +
		    		 Util.Round(autoResumeInt/1000,0) + "seconds.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: AutoResume enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Auto resume disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: AutoResume disabled.");
		} // endif
 	        break;

	    case "chkWarAvoid":
		flagWarAvoid = !flagWarAvoid;
		skapi.SetControlProperty(szPanelConfig, "chkWarAvoid", "checked", flagWarAvoid );
		if (flagWarAvoid) {
		    if (flagVerbose) Chat.Screen("War spell evasion enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: War spell evasion enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("War spell evasion disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: War spell evasion disabled.");
		} // endif
		break;

	    case "chkSlideCast":
		flagSlideCast = !flagSlideCast;
		skapi.SetControlProperty(szPanelConfig, "chkSlideCast", "checked", flagSlideCast );
		if (flagSlideCast) {
		    if (flagVerbose) Chat.Screen("Slide casting enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Slide casting enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Slide casting disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Slide casting disabled.");
		} // endif
		break;

	    case "chkRespondWhere":
		flagRespondWhere = !flagRespondWhere;
		skapi.SetControlProperty(szPanelConfig, "chkRespondWhere", "checked", flagRespondWhere );
		if (flagRespondWhere) {
		    if (flagVerbose) Chat.Screen("!whereto responding enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: !whereto responding enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("!whereto responding disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: !whereto responding disabled.");
		} // endif
		break;


	    // P(portal) 1 ******************************************

	    case "chkCloseBy":
		flagCloseOnly = !flagCloseOnly;
		skapi.SetControlProperty(szPanelConfig, "chkCloseBy", "checked", flagCloseOnly );
		if (flagCloseOnly) {
		    if (flagVerbose) Chat.Screen("Requestor must be closeby to issue commands.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Requestor must be closeby to issue commands.");
		} else {
		    if (flagVerbose) Chat.Screen("Requestor can be anywhere when issuing commands.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Requestor can be anywhere when issuing commands.");
		} // endif
		break;

	    case "chkPortalGems":
		flagPortalGems = !flagPortalGems;
		skapi.SetControlProperty(szPanelConfig, "chkPortalGems", "checked", flagPortalGems );
		if (flagPortalGems) {
		    if (flagVerbose) Chat.Screen("Use of portal gems enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Use of portal gems enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Use of portal gems disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Use of portal gems disabled.");
		} // endif
		BuildHelp();
		break;

	    case "chkAutoRotate":
		flagAutoRotate = !flagAutoRotate;
		skapi.SetControlProperty(szPanelConfig, "chkAutoRotate", "checked", flagAutoRotate );
		if (flagAutoRotate) {
		    if (flagVerbose) Chat.Screen("Turning before summoning enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Turning before summoning enabled.");
		} else {
		    Chat.Screen("Warning!!! Turning before summoning disabled. " + 
				"Portals may be created on top of each other!!!", cmcYellow );
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Turning before summoning disabled.");
		} // endif
		break;

	    case "chkAvoidStorms":
		flagStormAvoid = !flagStormAvoid;
		skapi.SetControlProperty(szPanelConfig, "chkAvoidStorms", "checked", flagStormAvoid);
		if (flagStormAvoid) {
		    if (flagVerbose) Chat.Screen("Portal storm avoiding enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Portal storm avoiding enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Portal storm avoiding disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Portal storm avoiding disabled.");
		} // endif
		break;

	    case "chkAdvertise":
		flagAdvertise = !flagAdvertise;
		skapi.SetControlProperty(szPanelConfig, "chkAdvertise", "checked", flagAdvertise);
		if (flagAdvertise) {
		    if (flagVerbose) Chat.Screen("Service advertising enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Service advertising enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Service advertising disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Service advertising disabled.");
		} // endif
		break;

	    case "chkOpenMsg":
		flagOpenMsg = !flagOpenMsg;
		skapi.SetControlProperty(szPanelConfig, "chkOpenMsg", "checked", flagOpenMsg);
		if (flagOpenMsg) {
		    if (flagVerbose) Chat.Screen("Message after portal summon enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Message after portal summon enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Message after portal summon.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Message after portal summon disabled.");
		} // endif
		break;

	    case "chkAckGifts":
		flagAckGifts = !flagAckGifts;
		skapi.SetControlProperty(szPanelConfig, "chkAckGifts", "checked", flagAckGifts );
		if (flagAckGifts) {
		    if (flagVerbose) Chat.Screen("Gift acknowledging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Gift acknowledging enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Gift acknowledging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Gift acknowledging disabled.");
		} // endif
		break;

	    case "chkLogGifts":
		flagLogGifts = !flagLogGifts;
		skapi.SetControlProperty(szPanelConfig, "chkLogGifts", "checked", flagLogGifts );
		if (flagLogGifts) {
		    if (flagVerbose) Chat.Screen("Gift logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Gift logging enabled.");
		    GiftLog = new LogFileObj(fileGiftLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("Gift logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Gift logging disabled.");
		    GiftLog.Close();
		} // endif
		break;

	    case "chkLogStats":
		flagLogStats = !flagLogStats;
		skapi.SetControlProperty(szPanelConfig, "chkLogStats", "checked", flagLogStats );
		if (flagLogStats) {
		    if (flagVerbose) Chat.Screen("Stats logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Stats logging enabled.");
		    StatLog = new LogFileObj(fileStatLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("Stats logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Stats logging disabled.");
		    StatLog.Close();
		} // endif
		break;

	    case "chkLogCmds":
		flagLogCmds = !flagLogCmds;
		skapi.SetControlProperty(szPanelConfig, "chkLogCmds", "checked", flagLogCmds );
		if (flagLogCmds) {
		    if (flagVerbose) Chat.Screen("Command logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Command logging enabled.");
		    CmdLog = new LogFileObj(fileCmdLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("Command logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Command logging disabled.");
		    CmdLog.Close();
		} // endif
		break;

	    case "chkAcceptLocal":
		flagAcceptLocal = !flagAcceptLocal;
		skapi.SetControlProperty(szPanelConfig, "chkAcceptLocal", "checked", flagAcceptLocal );
		if (flagAcceptLocal) {
		    if (flagVerbose) Chat.Screen("Local chat summon commands enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Local chat summon commands enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Local chat summon commands disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Local chat summon commands disabled.");
		} // endif
		break;


	    // P(portal) 2 ******************************************

   	    case "portalChar":
		// Update text for new buff selected
		var CharSlot = parseInt(dictSzValue("portalChar"));
		DisplayConfigPortal(CharSlot);
		break;

   	    case "healSelf":
   	        var CharSlot = parseInt(dictSzValue("portalChar"));
	    	var NewLvl = parseInt(dictSzValue("healSelf"));
	    	if (NewLvl == Toon[CharSlot].SpellHeal) break;
	    	if (!Cast.Know(spellListHeal[NewLvl])) {
		    Chat.Screen("You do not know Heal Self " + NewLvl + "!");
		    skapi.SetControlProperty(szPanelConfig, "healSelf", "selected", 7-Toon[CharSlot].SpellHeal);
		    break;
		} // endif
		Toon[CharSlot].SpellHeal = NewLvl;
		SpellObj_Heal = skapi.SpellInfoFromSpellid(spellListHeal[NewLvl]);
	    	if (flagVerbose) Chat.Screen("Heal Self spell level set to " + NewLvl + ".");
	  	if (logDecal||debugDecal) Log.OutD("Heal Self spell level set to " + NewLvl);

	  	// If we are in Restricted mode due to not knowing spells at macro start, then see if
	  	// we now know all spells needed for proper macro operation
	  	if (Restricted) {
		    if (SpellCheck(false)) {
		    	Chat.Screen("All spells necessary for macro operation are now known, Restricted flag cleared.");
		    	Restricted = false;
		    } else {
		    	Chat.Screen("One or more spells still not known, macro still in Restricted mode.");
		    } // endif
		} // endif
   	    	break;

   	    case "stam2mana":
		var CharSlot = parseInt(dictSzValue("portalChar"));
	    	var NewLvl = parseInt(dictSzValue("stam2mana"));
	    	if (NewLvl == Toon[CharSlot].SpellStamMana) break;
	    	if (!Cast.Know(spellListStamMana[NewLvl])) {
		    Chat.Screen("You do not know Stamina to Mana " + NewLvl + "!");
		    skapi.SetControlProperty(szPanelConfig, "stam2mana", "selected", 7-Toon[CharSlot].SpellStamMana);
		    break;
		} // endif
		Toon[CharSlot].SpellStamMana = NewLvl;
		SpellObj_StamMana = skapi.SpellInfoFromSpellid(spellListStamMana[NewLvl]);
	    	if (flagVerbose) Chat.Screen("Stamina to Mana spell level set to " + NewLvl + ".");
	  	if (logDecal||debugDecal) Log.OutD("Stamina to Mana spell level set to " + NewLvl);

	  	// If we are in Restricted mode due to not knowing spells at macro start, then see if
	  	// we now know all spells needed for proper macro operation
	  	if (Restricted) {
		    if (SpellCheck(false)) {
		    	Chat.Screen("All spells necessary for macro operation are now known, Restricted flag cleared.");
		    	Restricted = false;
		    } else {
		    	Chat.Screen("One or more spells still not known, macro still in Restricted mode.");
		    } // endif
		} // endif
   	   	break;

   	    case "revitSelf":
		var CharSlot = parseInt(dictSzValue("portalChar"));
	    	var NewLvl = parseInt(dictSzValue("revitSelf"));
	    	if (NewLvl == Toon[CharSlot].SpellRevitalize) break;
	    	if (!Cast.Know(spellListRevitalize[NewLvl])) {
		    Chat.Screen("You do not know Revitalize Self " + NewLvl + "!");
		    skapi.SetControlProperty(szPanelConfig, "revitSelf", "selected", 7-Toon[CharSlot].SpellRevitalize);
		    break;
		} // endif
		Toon[CharSlot].SpellRevitalize = NewLvl;
		SpellObj_Revitalize = skapi.SpellInfoFromSpellid(spellListRevitalize[NewLvl]);
	    	if (flagVerbose) Chat.Screen("Revitalize Self spell level set to " + NewLvl + ".");
	  	if (logDecal||debugDecal) Log.OutD("Revitalize Self spell level set to " + NewLvl);

	  	// If we are in Restricted mode due to not knowing spells at macro start, then see if
	  	// we now know all spells needed for proper macro operation
	  	if (Restricted) {
		    if (SpellCheck(false)) {
		    	Chat.Screen("All spells necessary for macro operation are now known, Restricted flag cleared.");
		    	Restricted = false;
		    } else {
		    	Chat.Screen("One or more spells still not known, macro still in Restricted mode.");
		    } // endif
		} // endif
		break;

   	    case "strengthSelf":
		var CharSlot = parseInt(dictSzValue("portalChar"));
	    	var NewLvl = parseInt(dictSzValue("strengthSelf"));
	    	if (NewLvl == Toon[CharSlot].SpellStrength) break;
	    	if (!Cast.Know(spellListStrength[NewLvl])) {
		    Chat.Screen("You do not know Strength Self " + NewLvl + "!");
		    skapi.SetControlProperty(szPanelConfig, "strengthSelf", "selected", 7-Toon[CharSlot].SpellStrength);
		    break;
		} // endif
		Toon[CharSlot].SpellStrength = NewLvl;
		SpellObj_Strength = skapi.SpellInfoFromSpellid(spellListStrength[NewLvl]);
	    	if (flagVerbose) Chat.Screen("Strength Self spell level set to " + NewLvl + ".");
	  	if (logDecal||debugDecal) Log.OutD("Strength Self spell level set to " + NewLvl);

	  	// If we are in Restricted mode due to not knowing spells at macro start, then see if
	  	// we now know all spells needed for proper macro operation
	  	if (Restricted) {
		    if (SpellCheck(false)) {
		    	Chat.Screen("All spells necessary for macro operation are now known, Restricted flag cleared.");
		    	Restricted = false;
		    } else {
		    	Chat.Screen("One or more spells still not known, macro still in Restricted mode.");
		    } // endif
		} // endif
		break;

   	    case "chkPrimEnabled":
   	        var CharSlot = parseInt(dictSzValue("portalChar"));
		Toon[CharSlot].plist[0].Enabled = !Toon[CharSlot].plist[0].Enabled;
		skapi.SetControlProperty(szPanelConfig, "chkPrimEnabled", "checked", Toon[CharSlot].plist[0].Enabled );
		if (Toon[CharSlot].plist[0].Enabled) {
		    if (Toon[CharSlot].plist[0].Keyword == null) {
			Chat.Screen("Can't enable portal with a null keyword");
			Toon[CharSlot].plist[0].Enabled = false;
			skapi.SetControlProperty(szPanelConfig, "chkPrimEnabled", "checked", false);
			return;
		    } // endif
		    if (flagVerbose)
			Chat.Screen("Primary portal summoning enabled for char " + skapi.rgszChar(CharSlot)+ ".");
		    if (logDecal||debugDecal)
			Log.OutD("OnControlEvent: Primary summoning enabled for char " + skapi.rgszChar(CharSlot)+ ".");
		} else {
		    if (flagVerbose)
			Chat.Screen("Primary portal summoning disabled for char " + skapi.rgszChar(CharSlot)+ ".");
		    if (logDecal||debugDecal)
			Log.OutD("OnControlEvent: Primary summoning disabled for char " + skapi.rgszChar(CharSlot)+ ".");
		} // endif
		break;

   	    case "editConfig4a":
		if (dictSzValue("editConfig4a") == null) {
		    szExp = null;
		} else {
		    szExp = dictSzValue("editConfig4a");
		} // endif
		if ((szExp == undefined) || (szExp == "")) szExp = null;
		
		var CharSlot = parseInt(dictSzValue("portalChar"));
		if (szExp == null) {
		    if (Toon[CharSlot].plist[0].Enabled) {
			Chat.Screen("Keyword is null, disabling portal");
			Toon[CharSlot].plist[0].Enabled = false;
			skapi.SetControlProperty(szPanelConfig, "chkPrimEnabled", "checked", false);
		    } // endif
		    Toon[CharSlot].plist[0].Keyword = null;
		    skapi.SetControlProperty(szPanelConfig, "text4av", "text", " ");
		} else {
		    Toon[CharSlot].plist[0].Keyword = szExp.toUpperCase();
		    skapi.SetControlProperty(szPanelConfig, "text4av", "text", Toon[CharSlot].plist[0].Keyword);
		} // endif
		break;

   	    case "editConfig4b":
		if (dictSzValue("editConfig4b") == null) {
		    szExp = null;
		} else {
		    szExp = dictSzValue("editConfig4b");
		} // endif
		if ((szExp == undefined) || (szExp == "")) szExp = null;
		var CharSlot = parseInt(dictSzValue("portalChar"));
		Toon[CharSlot].plist[0].Dest = szExp;
		if (szExp != null)
		    skapi.SetControlProperty(szPanelConfig, "text4bv", "text", Toon[CharSlot].plist[0].Dest);
		else
		    skapi.SetControlProperty(szPanelConfig, "text4bv", "text", " ");
		break;

   	    case "chkSecEnabled":
		var CharSlot = parseInt(dictSzValue("portalChar"));
		Toon[CharSlot].plist[1].Enabled = !Toon[CharSlot].plist[1].Enabled;
		skapi.SetControlProperty(szPanelConfig, "chkSecEnabled", "checked", Toon[CharSlot].plist[1].Enabled );
		if (Toon[CharSlot].plist[1].Enabled) {
		    if (Toon[CharSlot].plist[1].Keyword == null) {
			Chat.Screen("Can't enable portal with a null keyword");
			Toon[CharSlot].plist[1].Enabled = false;
			skapi.SetControlProperty(szPanelConfig, "chkSecEnabled", "checked", false);
			return;
		    } // endif
		    if (flagVerbose)
			Chat.Screen("Secondary portal summoning enabled for char " + skapi.rgszChar(CharSlot)+ ".");
		    if (logDecal||debugDecal)
			Log.OutD("OnControlEvent: Secondary summoning enabled for char " + skapi.rgszChar(CharSlot)+ ".");
		} else {
		    if (flagVerbose)
			Chat.Screen("Secondary portal summoning disabled for char " + skapi.rgszChar(CharSlot)+ ".");
		    if (logDecal||debugDecal)
			Log.OutD("OnControlEvent: Secondary summoning disabled for char " + skapi.rgszChar(CharSlot)+ ".");
		} // endif
		break;

   	    case "editConfig4e":
		if (dictSzValue("editConfig4e") == null) {
		    szExp = null;
		} else {
		    szExp = dictSzValue("editConfig4e");
		} // endif
		if ((szExp == undefined) || (szExp == "")) szExp = null;
		
		var CharSlot = parseInt(dictSzValue("portalChar"));
		if (szExp == null) {
		    if (Toon[CharSlot].plist[1].Enabled) {
			Chat.Screen("Keyword is null, disabling portal");
			Toon[CharSlot].plist[1].Enabled = false;
			skapi.SetControlProperty(szPanelConfig, "chkSecEnabled", "checked", false);
		    } // endif
		    Toon[CharSlot].plist[1].Keyword = null;
		    skapi.SetControlProperty(szPanelConfig, "text4ev", "text", " ");
		} else {
		    Toon[CharSlot].plist[1].Keyword = szExp.toUpperCase();
		    skapi.SetControlProperty(szPanelConfig, "text4ev", "text", Toon[CharSlot].plist[1].Keyword);
		} // endif
		break;

   	    case "editConfig4f":
		if (dictSzValue("editConfig4f") == null) {
		    szExp = null;
		} else {
		    szExp = dictSzValue("editConfig4f");
		} // endif
		if ((szExp == undefined) || (szExp == "")) szExp = null;
		var CharSlot = parseInt(dictSzValue("portalChar"));
		Toon[CharSlot].plist[1].Dest = szExp;
		if (szExp != null)
		    skapi.SetControlProperty(szPanelConfig, "text4fv", "text", Toon[CharSlot].plist[1].Dest);
		else
		    skapi.SetControlProperty(szPanelConfig, "text4fv", "text", " ");
		break;


	    // I(nfo) ******************************************

	    case "chkRepPlayer":
		flagReportPlayer = !flagReportPlayer;
		skapi.SetControlProperty(szPanelConfig, "chkRepPlayer", "checked", flagReportPlayer );
		if (flagReportPlayer) {
		    if (flagVerbose) Chat.Screen("Player reporting enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Player reporting disabled.");
		} // endif
		break;

	    case "chkChatSupport":
		flagChatSup = !flagChatSup;
		skapi.SetControlProperty(szPanelConfig, "chkChatSuport", "checked", flagChatSup);
		if (flagChatSup) {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to chat enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to chat disabled.");
		} // endif
		break;

	    case "chkAllegSupport":
		flagAllegSup = !flagAllegSup;
		skapi.SetControlProperty(szPanelConfig, "chkAllegSupport", "checked", flagAllegSup);
		if (flagAllegSup) {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to @alleg enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to @alleg disabled.");
		} // endif
		break;

	    case "chkFriendSupport":
		flagFriendSup = !flagFriendSup;
		skapi.SetControlProperty(szPanelConfig, "chkFriendSupport", "checked", flagFriendSup);
		if (flagFriendSup) {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to friends enabled.");
		    if (supportChars.length == 0) {
		    	Chat.Screen("WARNING! No characters listed in 'supportChars' variable to receive @tells");
		    } else {
			Chat.Screen("The following chars will receive @tells: " + supportChars.join());
		    } // endif
		} else {
		    if (flagVerbose) Chat.Screen("Spamming important macro events to friends disabled.");
		} // endif
		break;

	    case "chkLogChat":
		flagLogChat = !flagLogChat;
		skapi.SetControlProperty(szPanelConfig, "chkLogChat", "checked", flagLogChat );
		if (flagLogChat) {
		    if (flagVerbose) Chat.Screen("Local chat logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Local chat logging enabled.");
		    if (!(flagLogAlleg||flagLogGeneral||flagLogTrade))
			ChatLog = new LogFileObj(fileChatLog, ForAppending, true);	
		} else {
		    if (flagVerbose) Chat.Screen("Local chat logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Local chat logging disabled.");
		    if (!(flagLogAlleg||flagLogGeneral||flagLogTrade)) ChatLog.Close();
		} // endif
		break;

	    case "chkLogAlleg":
		flagLogAlleg = !flagLogAlleg;
		skapi.SetControlProperty(szPanelConfig, "chkLogAlleg", "checked", flagLogAlleg );
		if (flagLogAlleg) {
		    if (flagVerbose) Chat.Screen("Allegiance chat logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Allegiance chat logging enabled.");
		    if (!(flagLogChat||flagLogGeneral||flagLogTrade))
			ChatLog = new LogFileObj(fileChatLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("Allegiance chat logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Allegiance chat logging disabled.");
		    if (!(flagLogChat||flagLogGeneral||flagLogTrade)) ChatLog.Close();
		} // endif
		break;

	    case "chkLogGen":
		flagLogGeneral = !flagLogGeneral;
		skapi.SetControlProperty(szPanelConfig, "chkLogGen", "checked", flagLogGeneral);
		if (flagLogGeneral) {
		    if (flagVerbose) Chat.Screen("General chat logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: General chat logging enabled.");
		    if (!(flagLogChat||flagLogAlleg||flagLogTrade))
			ChatLog = new LogFileObj(fileChatLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("General chat logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: General chat logging disabled.");
		    if (!(flagLogChat||flagLogAlleg||flagLogTrade)) ChatLog.Close();
		} // endif
		break;

	    case "chkLogTrade":
		flagLogTrade = !flagLogTrade;
		skapi.SetControlProperty(szPanelConfig, "chkLogTrade", "checked", flagLogTrade);
		if (flagLogTrade) {
		    if (flagVerbose) Chat.Screen("Trade chat logging enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Trade chat logging enabled.");
		    if (!(flagLogChat||flagLogAlleg||flagLogGeneral))
			ChatLog = new LogFileObj(fileChatLog, ForAppending, true);
		} else {
		    if (flagVerbose) Chat.Screen("Trade chat logging disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Trade chat logging disabled.");
		    if (!(flagLogChat||flagLogAlleg||flagLogGeneral)) ChatLog.Close();
		} // endif
		break;

	    case "chkVerbose":
		flagVerbose = !flagVerbose;
		skapi.SetControlProperty(szPanelConfig, "chkVerbose", "checked", flagVerbose);
		if (flagVerbose) {
		    Chat.Screen("Verbose mode enabled.");
		} else {
		    Chat.Screen("Verbose mode disabled.");
		} // endif
		break;


	    // S(tartup) tab ***********************************

	    case "chkStartBuff":
		startBuffMode = !startBuffMode;
		skapi.SetControlProperty(szPanelConfig, "chkStartBuff", "checked", startBuffMode);
		if (startBuffMode) {
		    if (flagVerbose) Chat.Screen("Buffing will be enabled at macro start.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Buffing will be enabled at macro start.");
		} else {
		    if (flagVerbose) Chat.Screen("Buffing will be disabled at macro start.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Buffing will be disabled at macro start.");
		} // endif
		break;

	    case "chkStartSummon":
		startSummonMode = !startSummonMode;
		skapi.SetControlProperty(szPanelConfig, "chkStartSummon", "checked", startSummonMode);
		if (startSummonMode) {
		    if (flagVerbose) Chat.Screen("Summoning will be enabled at macro start.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Summoning enabled at macro start");
		} else {
		    Chat.Screen("WARNING!!! Summoning will be disabled at macro start.", cmcYellow);
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Summoning disabled at macro start");
		} // endif
		break;

	    case "chkStartFellow":
		creFellow = !creFellow;
		skapi.SetControlProperty(szPanelConfig, "chkStartFellow", "checked", creFellow);
		if (creFellow) {
		    Chat.Screen("Fellow '" + fellowName + "' will be created at macro start.");
		    if (logDecal||debugDecal)
			Log.OutD("OnControlEvent: Fellow '" + fellowName + "' will be created at macro start.");
		} else {
		    Chat.Screen("Fellow will not be created at macro start.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Fellow will not be created at macro start.");
		} // endif
		break;


	    // C(omponent) tab **************************************************

	    case "chkComps":
	    	flagCompCheck = !flagCompCheck
		skapi.SetControlProperty(szPanelConfig, "chkComps", "checked", flagCompCheck );
		if (flagCompCheck) {
		    if (flagVerbose) Chat.Screen("Component checking enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Component checking enabled");
		} else {
		    if (flagVerbose) Chat.Screen("WARNING!!! Component checking disabled.", cmcYellow);
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Component checking disabled");
		} // endif
		break;

 	    case "chkCompLog":
 	    	flagCompLogOff = !flagCompLogOff;
		skapi.SetControlProperty(szPanelConfig, "chkCompLog", "checked", flagCompLogOff );
		if (flagCompLogOff) {
		    if (flagVerbose) Chat.Screen("Logoff when comps low enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff when comps low enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Logoff when comps low disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Logoff when comps low disabled.");
		} // endif
 	        break;

	    case "chkCompAssist":
 	    	flagCompAssist = !flagCompAssist;
		skapi.SetControlProperty(szPanelConfig, "chkCompAssist", "checked", flagCompAssist);
		if (flagCompAssist) {
		    if (flagVerbose) Chat.Screen("Request for comps when low enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Request for comps when low enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Request for comps when low disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Request for comps when low disabled.");
		} // endif
 	        break;

	    case "chkSplitPeas":
 	    	flagSplitPeas = !flagSplitPeas;
		skapi.SetControlProperty(szPanelConfig, "chkSplitPeas", "checked", flagSplitPeas);
		if (flagSplitPeas) {
		    if (flagVerbose) Chat.Screen("Pea splitting enabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Pea splitting enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Pea splitting disabled.");
		    if (logDecal||debugDecal) Log.OutD("OnControlEvent: Pea splitting disabled.");
		} // endif
 	        break;


	    // A(lert) tab **************************************************

	    case "chkDeathAlert":
	    	alertDeath = !alertDeath;
		skapi.SetControlProperty(szPanelConfig, "chkDeathAlert", "checked", alertDeath);
		if (alertDeath) {
		   if (flagVerbose)  Chat.Screen("Death alert enabled.");
		    if (logDecal||debugDecal) Log.OutD("Death alert enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Death alert disabled.");
		    if (logDecal||debugDecal) Log.OutD("Death alert disabled.");
		} // endif
		break;

	    case "chkHideAlert":
	    	alertHide = !alertHide;
		skapi.SetControlProperty(szPanelConfig, "chkHideAlert", "checked", alertHide);
		if (alertHide) {
		   if (flagVerbose)  Chat.Screen("Hide alert enabled.");
		    if (logDecal||debugDecal) Log.OutD("Hide alert enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Hide alert disabled.");
		    if (logDecal||debugDecal) Log.OutD("Hide alert disabled.");
		} // endif
		break;

	    case "chkCompAlert":
	    	alertComps = !alertComps;
		skapi.SetControlProperty(szPanelConfig, "chkCompAlert", "checked", alertComps);
		if (alertComps) {
		    if (flagVerbose) Chat.Screen("Low component alert enabled.");
		    if (logDecal||debugDecal) Log.OutD("Low component alert enabled");
		} else {
		    if (flagVerbose) Chat.Screen("Low component alert disabled.");
		    if (logDecal||debugDecal) Log.OutD("Low component alert disabled.");
		} // endif
		break;

	    case "chkAbortAlert":
	    	alertAbort = !alertAbort;
		skapi.SetControlProperty(szPanelConfig, "chkAbortAlert", "checked", alertAbort);
		if (alertAbort) {
		    if (flagVerbose) Chat.Screen("Abort alert enabled.");
		    if (logDecal||debugDecal) Log.OutD("Abort alert enabled.");
		} else {
		    if (flagVerbose) Chat.Screen("Abort alert disabled.");
		    if (logDecal||debugDecal) Log.OutD("Abort alert disabled.");
		} // endif
		break;

    	    case "btnTestAlert":
	    	alertDeathWav.Play();
	        break;


	    // D(ebug) config tab *************************

	    case "chkDebug":
		debug = !debug;
		skapi.SetControlProperty(szPanelConfig, "chkDebug", "checked", debug);
		if (debug) {
		    if (flagVerbose) Chat.Screen("General debug info logging turned on.");
		    Log.OutD("General debug info logging turned on.");
		} else {
		    if (flagVerbose) Chat.Screen("General debug info logging turned off.");
		    Log.OutD("General debug info logging turned off.");
		} // endif
		break;

	    case "chkEvents":
	    	debugEvent = !debugEvent;
		skapi.SetControlProperty(szPanelConfig, "chkEvents", "checked", debugEvent);
		if (debugEvent) {
		    if (flagVerbose) Chat.Screen("Event debug info logging turned on.");
		    Log.OutD("Event debug info logging turned on.");
		} else {
		    if (flagVerbose) Chat.Screen("Event debug info logging turned off.");
		    Log.OutD("Event debug info logging turned off.");
		} // endif
		break;

	    case "chkStateDebug":
		debugState = !debugState;
		skapi.SetControlProperty(szPanelConfig, "chkStateDebug", "checked", debugState);
		if (debugState) {
		    if (flagVerbose) Chat.Screen("State debug info logging turned on.");
		    Log.OutD("State debug info logging turned on.");
		} else {
		    if (flagVerbose) Chat.Screen("State debug info logging turned off.");
		    Log.OutD("State debug info logging turned off.");
		} // endif
		break;

	    case "chkIdleDebug":
		debugIdle = !debugIdle;
		skapi.SetControlProperty(szPanelConfig, "chkIdleDebug", "checked", debugIdle);
		if (debugIdle) {
		    if (flagVerbose) Chat.Screen("Idle debug info logging turned on.");
		    Log.OutD("Idle debug info logging turned on.");
		} else {
		    if (flagVerbose) Chat.Screen("Idle debug info logging turned off.");
		    Log.OutD("Idle debug info logging turned off.");
		} // endif
		break;

	    case "chkBuffDebug":
		debugBuffs = !debugBuffs;
		skapi.SetControlProperty(szPanelConfig, "chkBuffDebug", "checked", debugBuffs);
		if (debugBuffs) {
		    Buff[AccCurChar].SetDebug(true);
		    if (flagVerbose) Chat.Screen("Buff debug info logging turned on.");
		    Log.OutD("Buff debug info logging turned on.");
		} else {
		    Buff[AccCurChar].SetDebug(false);
		    if (flagVerbose) Chat.Screen("Buff debug info logging turned off.");
		    Log.OutD("Buff debug info logging turned off.");
		} // endif
		break;

	    case "chkCastDebug":
		debugCast = !debugCast;
		skapi.SetControlProperty(szPanelConfig, "chkCastDebug", "checked", debugCast);
		if (debugCast) {
		    Cast.SetDebug(true);
		    if (flagVerbose) Chat.Screen("Cast debug info logging turned on.");
		    Log.OutD("Cast debug info logging turned on.");
		} else {
		    Cast.SetDebug(false);
		    if (flagVerbose) Chat.Screen("Cast debug info logging turned off.");
		    Log.OutD("Cast debug info logging turned off.");
		} // endif
		break;

	    case "chkSummonDebug":
		debugSummon = !debugSummon;
		skapi.SetControlProperty(szPanelConfig, "chkSummonDebug", "checked", debugSummon);
		if (debugSummon) {
		    if (flagVerbose) Chat.Screen("Summon debug info logging turned on.");
		    Log.OutD("Summon debug info logging turned on.");
		} else {
		    if (flagVerbose) Chat.Screen("Summon debug info logging turned off.");
		    Log.OutD("Summon debug info logging turned off.");
		} // endif
		break;
	} // end switch
	if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: End, Config");
    	return;
    } // endif


    // BUFF PANEL **************************************************************************************************

    if (szPanel == szPanelBuffs) {
	// Process Buff Panel Controls
	switch (szControl) {
	    // Gen(eral) Buff Section *****************************

	    case "btnBuffLoad":
	       	Chat.Screen("Loading buff configuration info...");
	        var Count = Buff[AccCurChar].Load(AccCurChar);
	        if (Count < 0) {
	            Chat.Screen("File " + Buff[AccCurChar].FileName + " not found!");
	            break;
	        } // endif
		DisplayBuffHerm();
		DisplayBuffDef();
		BuffIndex = parseInt(dictSzValue("creChoice"));
		DisplayBuffCre(BuffIndex);
		BuffIndex = parseInt(dictSzValue("lifeChoice"));
		DisplayBuffLife(BuffIndex);
		BuffIndex = parseInt(dictSzValue("armorChoice"));
		DisplayBuffArmor(BuffIndex);
	        Chat.Screen(Count + " active buffs read from file " + Buff[AccCurChar].FileName);
	        break;

	    case "btnBuffSave":
	       	Chat.Screen("Saving buff configuration info...");
		var SavedCount = Buff[AccCurChar].Save(AccCurChar);
		Chat.Screen("Buff information saved.  " + SavedCount + " active buffs written to file " +
			     Buff[AccCurChar].FileName);
	        break;

	    case "btnBuffReset":
	        // Reset buffs to factory defaults
	        Chat.Screen("Resetting...");
	        Buff[AccCurChar].Reset();
		DisplayBuffHerm();
		DisplayBuffDef();
		BuffIndex = parseInt(dictSzValue("creChoice"));
		DisplayBuffCre(BuffIndex);
		BuffIndex = parseInt(dictSzValue("lifeChoice"));
		DisplayBuffLife(BuffIndex);
		BuffIndex = parseInt(dictSzValue("armorChoice"));
		DisplayBuffArmor(BuffIndex);
		Chat.Screen("All buffs reset to default settings!");
	        break;

	    case "btnBuffShow":
	        if (!Buff[AccCurChar].Show()) Chat.Screen("No buffs enabled!");
	        break;


	    // Cre(ature) Buff Section ****************************************************

    	    case "creChoice":
		// Update text for new buff selected
		BuffCrePrev = BuffCreIndex;
		BuffCreIndex = parseInt(dictSzValue("creChoice"));
		DisplayBuffCre(BuffCreIndex);
		break;

	    case "chkBuffCreEnab":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		var Level = Buff[AccCurChar].Info[BuffIndex].Level;
		if (!Buff[AccCurChar].Info[BuffIndex].Enabled && !Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Level])) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff not enabled.");
		    skapi.SetControlProperty(szPanelBuffs, "chkBuffCreEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled = !Buff[AccCurChar].Info[BuffIndex].Enabled;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffCreEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		BuffCreLast = BuffIndex;
		break;

	    case "chkBuffCreRandom":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		Buff[AccCurChar].Info[BuffIndex].Randomize = !Buff[AccCurChar].Info[BuffIndex].Randomize;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffCreRandom", "checked", Buff[AccCurChar].Info[BuffIndex].Randomize);
		BuffCreLast = BuffIndex;
		break;

	    case "choiceBuff2b":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		var BuffLevel = parseInt(dictSzValue("choiceBuff2b"));
		if (BuffLevel == Buff[AccCurChar].Info[BuffIndex].Level) break;
	    	if (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel])) {
		    Chat.Screen("WARNING!!! You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " + BuffLevel + ".");
		    if (Buff[AccCurChar].Info[BuffIndex].Enabled) {
			Chat.Screen("WARNING!!! Buff disabled");
			Buff[AccCurChar].Info[BuffIndex].Enabled = false;
			skapi.SetControlProperty(szPanelBuffs, "chkBuffCreEnab", "checked", false);
		    } // endif
		} // endif
		Buff[AccCurChar].Info[BuffIndex].SpellObj = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel]);
		Buff[AccCurChar].Info[BuffIndex].Level = BuffLevel;
		BuffCreLast = BuffIndex;
		break;

	    case "editBuff2c":
		szExp = dictSzValue("editBuff2c");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Safety = intExp*1000;
		skapi.SetControlProperty(szPanelBuffs, "text2cv", "text", intExp);
		BuffCreLast = BuffIndex;
		break;

	    case "editBuff2d":
		szExp = dictSzValue("editBuff2d");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Priority = intExp;
		skapi.SetControlProperty(szPanelBuffs, "text2dv", "text", intExp);
		BuffCreLast = BuffIndex;
		break;

	    case "btnBuffCreNext":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		BuffIndex++;
		if (BuffIndex > Buff[AccCurChar].Range[buffTypeCre].End) BuffIndex = Buff[AccCurChar].Range[buffTypeCre].Begin;
		skapi.SetControlProperty(szPanelBuffs, "creChoice", "selected", BuffIndex-Buff[AccCurChar].Range[buffTypeCre].Begin);
		break;

	    case "btnBuffCrePrev":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		if (BuffIndex == Buff[AccCurChar].Range[buffTypeCre].Begin)
		    BuffIndex = Buff[AccCurChar].Range[buffTypeCre].End;
		else
		    BuffIndex--;
		skapi.SetControlProperty(szPanelBuffs, "creChoice", "selected", BuffIndex-Buff[AccCurChar].Range[buffTypeCre].Begin);
		break;

	    case "btnBuffCreSetPrev":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		if (BuffIndex == BuffCrePrev) {
		    Chat.Screen("Previous examined creature buff is same as current, aborted.");
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffCrePrev].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffCrePrev].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffCrePrev].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffCrePrev].Safety;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffCre(BuffIndex);
		BuffCreLast = BuffIndex;
		if (flagVerbose) Chat.Screen("Current buff set same as previous creature buff examined.");
		break;

	    case "btnBuffCreSetLast":
	        if (BuffCreLast < 0)  {
		    Chat.Screen("No creature buffs modified since macro start, can't set as last modified!");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		if (BuffIndex == BuffCreLast) {
		    Chat.Screen("Last modified creature buff is same as current, aborted.");
		    break;
		} // endif

		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffCreLast].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffCreLast].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffCreLast].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffCreLast].Safety;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffCre(BuffIndex);
		BuffCreLast = BuffIndex;
		if (flagVerbose) Chat.Screen("Current buff set same as last modified creature buff.");
		break;

	    case "btnBuffCreAll":
		var BuffIndex = parseInt(dictSzValue("creChoice"));
		for (var x=Buff[AccCurChar].Range[buffTypeCre].Begin; x<=Buff[AccCurChar].Range[buffTypeCre].End; x++) {
		    if (x == BuffIndex) continue;
		    Buff[AccCurChar].Info[x].Enabled    = Buff[AccCurChar].Info[BuffIndex].Enabled;
		    Buff[AccCurChar].Info[x].Level	    = Buff[AccCurChar].Info[BuffIndex].Level;
		    Buff[AccCurChar].Info[x].Randomize  = Buff[AccCurChar].Info[BuffIndex].Randomize;
		    Buff[AccCurChar].Info[x].Safety	    = Buff[AccCurChar].Info[BuffIndex].Safety;
		    Buff[AccCurChar].Info[x].SpellObj   = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]);

		    if ((Buff[AccCurChar].Info[x].Enabled) &&
			(!Cast.Know(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]))) {
			Chat.Screen("You do not know " + Buff[AccCurChar].Info[x].Name + " Self " +
		    		Buff[AccCurChar].Info[x].Level + ", buff disabled.");
			Buff[AccCurChar].Info[x].Enabled = false;
		    } // endif
		} // endfor
		if (flagVerbose) Chat.Screen("All creature buffs set like this one.");
		break;

	    case "btnBuffCreShow":
		if (!Buff[AccCurChar].Show(buffTypeCre)) Chat.Screen("No creature buffs enabled!");
		break;

	    // Life Buff Section ****************************************************

    	    case "lifeChoice":
		// Update text for new buff selected
		BuffLifePrev = BuffLifeIndex;
		BuffLifeIndex = parseInt(dictSzValue("lifeChoice"));
		DisplayBuffLife(BuffLifeIndex);
		break;

	    case "chkBuffLifeEnab":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		var Level = Buff[AccCurChar].Info[BuffIndex].Level;
		if (!Buff[AccCurChar].Info[BuffIndex].Enabled && !Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Level])) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff not enabled.");
		    skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled = !Buff[AccCurChar].Info[BuffIndex].Enabled;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		BuffLifeLast = BuffIndex;
		break;

	    case "chkBuffLifeRandom":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		Buff[AccCurChar].Info[BuffIndex].Randomize = !Buff[AccCurChar].Info[BuffIndex].Randomize;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeRandom", "checked", Buff[AccCurChar].Info[BuffIndex].Randomize);
		BuffLifeLast = BuffIndex;
		break;

	    case "choiceBuff3b":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		var BuffLevel = parseInt(dictSzValue("choiceBuff3b"));
		if (BuffLevel == Buff[AccCurChar].Info[BuffIndex].Level) break;
	    	if (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel])) {
		    Chat.Screen("WARNING!!! You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " + BuffLevel + ".");
		    if (Buff[AccCurChar].Info[BuffIndex].Enabled) {
			Chat.Screen("WARNING!!! Buff disabled!");
			Buff[AccCurChar].Info[BuffIndex].Enabled = false;
			skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeEnab", "checked", false);
		    } // endif
		} // endif
		Buff[AccCurChar].Info[BuffIndex].SpellObj = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel]);
		Buff[AccCurChar].Info[BuffIndex].Level = BuffLevel;
		BuffLifeLast = BuffIndex;
		break;

	    case "editBuff3c":
		szExp = dictSzValue("editBuff3c");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Safety = intExp*1000;
		skapi.SetControlProperty(szPanelBuffs, "text3cv", "text", intExp);
		BuffLifeLast = BuffIndex;
		break;

	    case "editBuff3d":
		szExp = dictSzValue("editBuff3d");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Priority = intExp;
		skapi.SetControlProperty(szPanelBuffs, "text3dv", "text", intExp);
		BuffLifeLast = BuffIndex;
		break;

	    case "btnBuffLifeNext":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		BuffIndex++;
		if (BuffIndex > Buff[AccCurChar].Range[buffTypeLife].End) BuffIndex = Buff[AccCurChar].Range[buffTypeLife].Begin;
		skapi.SetControlProperty(szPanelBuffs, "lifeChoice", "selected", BuffIndex-Buff[AccCurChar].Range[buffTypeLife].Begin);
		break;

	    case "btnBuffLifePrev":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		if (BuffIndex == Buff[AccCurChar].Range[buffTypeLife].Begin)
		    BuffIndex = Buff[AccCurChar].Range[buffTypeLife].End;
		else
		    BuffIndex--;
		skapi.SetControlProperty(szPanelBuffs, "lifeChoice", "selected", BuffIndex-Buff[AccCurChar].Range[buffTypeLife].Begin);
		break;

	    case "btnBuffLifeSetPrev":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		if (BuffIndex == BuffLifePrev) {
		    Chat.Screen("Previous examined buff is same as current, aborted.");
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffLifePrev].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffLifePrev].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffLifePrev].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffLifePrev].Safety;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffLife(BuffIndex);
		BuffLifeLast = BuffIndex;
		if (flagVerbose) Chat.Screen("Current buff set same as previous life buff examined.");
		break;

	    case "btnBuffLifeSetLast":
	        if (BuffLifeLast < 0)  {
		    Chat.Screen("No life buffs modified since macro start, can't set as last modified!");
		    break;
		} // endif

		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		if (BuffIndex == BuffLifeLast) {
		    Chat.Screen("Last modified life buff is same as current, aborted.");
		    break;
		} // endif

		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffLifeLast].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffLifeLast].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffLifeLast].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffLifeLast].Safety;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffLife(BuffIndex);
		BuffLifeLast = BuffIndex;
		if (flagVerbose) Chat.Screen("Current buff set same as last life buff modified.");
		break;

	    case "btnBuffLifeAll":
		var BuffIndex = parseInt(dictSzValue("lifeChoice"));
		for (var x=Buff[AccCurChar].Range[buffTypeLife].Begin; x<=Buff[AccCurChar].Range[buffTypeLife].End; x++) {
		    if (x == BuffIndex) continue;
		    Buff[AccCurChar].Info[x].Enabled    = Buff[AccCurChar].Info[BuffIndex].Enabled;
		    Buff[AccCurChar].Info[x].Level	    = Buff[AccCurChar].Info[BuffIndex].Level;
		    Buff[AccCurChar].Info[x].Randomize  = Buff[AccCurChar].Info[BuffIndex].Randomize;
		    Buff[AccCurChar].Info[x].Safety	    = Buff[AccCurChar].Info[BuffIndex].Safety;
		    Buff[AccCurChar].Info[x].SpellObj   = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]);

		    if ((Buff[AccCurChar].Info[x].Enabled) &&
			(!Cast.Know(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]))) {
			Chat.Screen("You do not know " + Buff[AccCurChar].Info[x].Name + " Self " +
		    		Buff[AccCurChar].Info[x].Level + ", buff disabled.");
			Buff[AccCurChar].Info[x].Enabled = false;
		    } // endif
		} // endfor
		if (flagVerbose) Chat.Screen("All life buffs set like this one.");
		break;

	    case "btnBuffLifeShow":
		if (!Buff[AccCurChar].Show(buffTypeLife)) Chat.Screen("No life buffs enabled!");
		break;

	    // Armor Buff Section ****************************************************

    	    case "armorChoice":
		// Update text for new buff selected
		BuffArmorPrev = BuffArmorIndex;
		BuffArmorIndex = parseInt(dictSzValue("armorChoice"));
		DisplayBuffArmor(BuffArmorIndex);
		break;

	    case "chkBuffArmorEnab":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		var Level = Buff[AccCurChar].Info[BuffIndex].Level;
		if (!Buff[AccCurChar].Info[BuffIndex].Enabled && !Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Level])) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff not enabled.");
		    skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled = !Buff[AccCurChar].Info[BuffIndex].Enabled;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorEnab", "checked", Buff[AccCurChar].Info[BuffIndex].Enabled);
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffArmorRandom":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		Buff[AccCurChar].Info[BuffIndex].Randomize = !Buff[AccCurChar].Info[BuffIndex].Randomize;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorRandom", "checked", Buff[AccCurChar].Info[BuffIndex].Randomize);
		BuffArmorLast = BuffIndex;
		break;

	    case "choiceBuff4b":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		var BuffLevel = parseInt(dictSzValue("choiceBuff4b"));
		if (BuffLevel == Buff[AccCurChar].Info[BuffIndex].Level) break;
	    	if (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel])) {
		    Chat.Screen("WARNING!!! You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " + BuffLevel + ".");
		    if (Buff[AccCurChar].Info[BuffIndex].Enabled) {
			Chat.Screen("WARNING!!! Buff disabled!");
			Buff[AccCurChar].Info[BuffIndex].Enabled = false;
			skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorEnab", "checked", false);
		    } // endif
		} // endif
		Buff[AccCurChar].Info[BuffIndex].SpellObj = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[BuffLevel]);
		Buff[AccCurChar].Info[BuffIndex].Level = BuffLevel;
		BuffArmorLast = BuffIndex;
		break;

	    case "editBuff4c":
		szExp = dictSzValue("editBuff4c");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Safety = intExp*1000;
		skapi.SetControlProperty(szPanelBuffs, "text4cv", "text", Buff[AccCurChar].Info[BuffIndex].Safety/1000);
		BuffArmorLast = BuffIndex;
		break;

	    case "editBuff4d":
		szExp = dictSzValue("editBuff4d");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffIndex].Priority = intExp;
		skapi.SetControlProperty(szPanelBuffs, "text4dv", "text", Buff[AccCurChar].Info[BuffIndex].Priority);
		BuffArmorLast = BuffIndex;
		break;

	    case "chkPaperDoll":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmPaperDoll) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmPaperDoll;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmPaperDoll;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkPaperDoll", "checked", 
					 (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmPaperDoll));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffHead":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmHead) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmHead;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmHead;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffHead", "checked", (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmHead));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffHands":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmHands) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmHands;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmHands;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffHands", "checked", (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmHands));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffFeet":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmFeet) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmFeet;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmFeet;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffFeet", "checked", (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmFeet));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffChest":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmChestOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmChestOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmChestOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffChest", "checked",
		    	(Buff[AccCurChar].Info[BuffIndex].Eqm & eqmChestOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffUpArms":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperArmsOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmUpperArmsOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmUpperArmsOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffUpArms", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperArmsOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffLowArms":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmLowerArmsOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmLowerArmsOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmLowerArmsOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffLowArms", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmLowerArmsOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffAbdomen":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmAbdomenOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmAbdomenOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmAbdomenOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffAbdomen", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmAbdomenOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffUpLegs":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperLegsOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmUpperLegsOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmUpperLegsOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffUpLegs", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperLegsOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffLowLegs":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmLowerLegsOuter) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmLowerLegsOuter;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmLowerLegsOuter;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffLowLegs", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmLowerLegsOuter));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffShirt":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmChestUnder) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmChestUnder;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmChestUnder;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffShirt", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmChestUnder));
		BuffArmorLast = BuffIndex;
		break;

	    case "chkBuffPants":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperLegsUnder) {
		    Buff[AccCurChar].Info[BuffIndex].Eqm &= ~eqmUpperLegsUnder;
		} else {
		    Buff[AccCurChar].Info[BuffIndex].Eqm |= eqmUpperLegsUnder;
		} // endif
		skapi.SetControlProperty(szPanelBuffs, "chkBuffPants", "checked",
		    (Buff[AccCurChar].Info[BuffIndex].Eqm & eqmUpperLegsUnder));
		BuffArmorLast = BuffIndex;
		break;

	    case "btnBuffArmorNext":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		BuffIndex++;
		if (BuffIndex > Buff[AccCurChar].ClassRange[equipClassArmor].End) BuffIndex = Buff[AccCurChar].ClassRange[equipClassArmor].Begin;
		skapi.SetControlProperty(szPanelBuffs, "armorChoice", "selected",
		    BuffIndex-Buff[AccCurChar].Range[buffTypeItem].Begin);
		break;

	    case "btnBuffArmorPrev":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		BuffIndex--;
		if (BuffIndex < Buff[AccCurChar].ClassRange[equipClassArmor].Begin) BuffIndex = Buff[AccCurChar].ClassRange[equipClassArmor].End;  
		skapi.SetControlProperty(szPanelBuffs, "armorChoice", "selected",
		    BuffIndex-Buff[AccCurChar].Range[buffTypeItem].Begin);
		break;

	    case "btnBuffArmorSetPrev":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (BuffIndex == BuffArmorPrev) {
		    Chat.Screen("Previous examined armor buff is same as current, aborted.");
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffArmorPrev].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffArmorPrev].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffArmorPrev].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffArmorPrev].Safety;
		Buff[AccCurChar].Info[BuffIndex].Eqm	= Buff[AccCurChar].Info[BuffArmorPrev].Eqm;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffArmor(BuffIndex);
		if (flagVerbose) Chat.Screen("Current buff set same as previous item armor buff examined.");
		break;

	    case "btnBuffArmorSetLast":
	        if (BuffArmorLast < 0)  {
		    Chat.Screen("No item armor buffs modified since macro start, can't set as last modified!");
		    break;
		} // endif

		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		if (BuffIndex == BuffArmorLast) {
		    Chat.Screen("Last modified item armor buff is same as current, aborted.");
		    break;
		} // endif

		Buff[AccCurChar].Info[BuffIndex].Enabled	= Buff[AccCurChar].Info[BuffArmorLast].Enabled;
		Buff[AccCurChar].Info[BuffIndex].Level	= Buff[AccCurChar].Info[BuffArmorLast].Level;
		Buff[AccCurChar].Info[BuffIndex].Randomize	= Buff[AccCurChar].Info[BuffArmorLast].Randomize;
		Buff[AccCurChar].Info[BuffIndex].Safety	= Buff[AccCurChar].Info[BuffArmorLast].Safety;
		Buff[AccCurChar].Info[BuffIndex].Eqm	= Buff[AccCurChar].Info[BuffArmorLast].Eqm;
		Buff[AccCurChar].Info[BuffIndex].SpellObj	=
		    skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]);

	    	if ((Buff[AccCurChar].Info[BuffIndex].Enabled) &&
		    (!Cast.Know(Buff[AccCurChar].Info[BuffIndex].Self[Buff[AccCurChar].Info[BuffIndex].Level]))) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffIndex].Name + " Self " +
		    		Buff[AccCurChar].Info[BuffIndex].Level + ", buff disabled.");
		    Buff[AccCurChar].Info[BuffIndex].Enabled = false;
		} // endif
		DisplayBuffArmor(BuffIndex);
		if (flagVerbose) Chat.Screen("Current buff set same as last item armor buff modified.");
		break;

	    case "btnBuffArmorAll":
		var BuffIndex = parseInt(dictSzValue("armorChoice"));
		for (var x=Buff[AccCurChar].ClassRange[equipClassArmor].Begin; x<=Buff[AccCurChar].ClassRange[equipClassArmor].End; x++) {
		    if (x == BuffIndex) continue;
		    Buff[AccCurChar].Info[x].Enabled    = Buff[AccCurChar].Info[BuffIndex].Enabled;
		    Buff[AccCurChar].Info[x].Level	    = Buff[AccCurChar].Info[BuffIndex].Level;
		    Buff[AccCurChar].Info[x].Randomize  = Buff[AccCurChar].Info[BuffIndex].Randomize;
		    Buff[AccCurChar].Info[x].Safety	    = Buff[AccCurChar].Info[BuffIndex].Safety;
		    Buff[AccCurChar].Info[x].Eqm	    = Buff[AccCurChar].Info[BuffIndex].Eqm;
		    Buff[AccCurChar].Info[x].SpellObj   = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]);

		    if ((Buff[AccCurChar].Info[x].Enabled) &&
			(!Cast.Know(Buff[AccCurChar].Info[x].Self[Buff[AccCurChar].Info[x].Level]))) {
			Chat.Screen("You do not know " + Buff[AccCurChar].Info[x].Name + " Self " +
		    		Buff[AccCurChar].Info[x].Level + ", buff disabled.");
			Buff[AccCurChar].Info[x].Enabled = false;
		    } // endif
		} // endfor
		if (flagVerbose) Chat.Screen("All item armor buffs set like this one.");
		break;

	    case "btnBuffArmorShow":
		if (!Buff[AccCurChar].Show(buffTypeItem,equipClassArmor)) Chat.Screen("No item buffs enabled!");
		break;

	    // Wand Buff Section ****************************************************

	    case "chkBuffHermEnab":
		var Level = Buff[AccCurChar].Info[BuffHermIndex].Level;
		if (!Buff[AccCurChar].Info[BuffHermIndex].Enabled && !Cast.Know(Buff[AccCurChar].Info[BuffHermIndex].Self[Level])) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffHermIndex].Name + " Self " +
		    		 Level + ", buff not enabled.");
		    skapi.SetControlProperty(szPanelBuffs, "chkBuffHermEnab", "checked",
		    			     Buff[AccCurChar].Info[BuffHermIndex].Enabled);
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffHermIndex].Enabled = !Buff[AccCurChar].Info[BuffHermIndex].Enabled;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffHermEnab", "checked", Buff[AccCurChar].Info[BuffHermIndex].Enabled);
		break;

	    case "chkBuffHermRandom":
		Buff[AccCurChar].Info[BuffHermIndex].Randomize = !Buff[AccCurChar].Info[BuffHermIndex].Randomize;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffHermRandom", "checked",
					 Buff[AccCurChar].Info[BuffHermIndex].Randomize);
		break;

	    case "choiceBuff1b":
		var BuffLevel = parseInt(dictSzValue("choiceBuff1b"));
		if (BuffLevel == Buff[AccCurChar].Info[BuffHermIndex].Level) break;
	    	if (!Cast.Know(Buff[AccCurChar].Info[BuffHermIndex].Self[BuffLevel])) {
		    Chat.Screen("WARNING!!! You don't know " + Buff[AccCurChar].Info[BuffHermIndex].Name + " Self " + BuffLevel + ".");
		    if (Buff[AccCurChar].Info[BuffHermIndex].Enabled) {
			Chat.Screen("WARNING!!! Buff disabled!");
			Buff[AccCurChar].Info[BuffHermIndex].Enabled = false;
			skapi.SetControlProperty(szPanelBuffs, "chkBuffHermEnab", "checked", false);
			break;
		    } // endif
		} // endif
		Buff[AccCurChar].Info[BuffHermIndex].SpellObj = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffHermIndex].Self[BuffLevel]);
		Buff[AccCurChar].Info[BuffHermIndex].Level = BuffLevel;
		break;

	    case "editBuff1c":
		szExp = dictSzValue("editBuff1c");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffHermIndex].Safety = intExp*1000;
		skapi.SetControlProperty(szPanelBuffs, "text1cv", "text", Buff[AccCurChar].Info[BuffHermIndex].Safety/1000);
		break;

	    case "editBuff1d":
		szExp = dictSzValue("editBuff1d");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffHermIndex].Priority = intExp;
		skapi.SetControlProperty(szPanelBuffs, "text1dv", "text", Buff[AccCurChar].Info[BuffHermIndex].Priority);
		break;

	    // Defender on wand

	    case "chkBuffDefEnab":
		var Level = Buff[AccCurChar].Info[BuffDefIndex].Level;
		if (!Buff[AccCurChar].Info[BuffDefIndex].Enabled && !Cast.Know(Buff[AccCurChar].Info[BuffDefIndex].Self[Level])) {
		    Chat.Screen("You do not know " + Buff[AccCurChar].Info[BuffDefIndex].Name + " Self " +
		    		 Level + ", buff not enabled.");
		    skapi.SetControlProperty(szPanelBuffs, "chkBuffDefEnab", "checked",
		    			     Buff[AccCurChar].Info[BuffDefIndex].Enabled);
		    break;
		} // endif
		Buff[AccCurChar].Info[BuffDefIndex].Enabled = !Buff[AccCurChar].Info[BuffDefIndex].Enabled;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffDefEnab", "checked", Buff[AccCurChar].Info[BuffDefIndex].Enabled);
		break;

	    case "chkBuffDefRandom":
		Buff[AccCurChar].Info[BuffDefIndex].Randomize = !Buff[AccCurChar].Info[BuffDefIndex].Randomize;
		skapi.SetControlProperty(szPanelBuffs, "chkBuffDefRandom", "checked",
					 Buff[AccCurChar].Info[BuffDefIndex].Randomize);
		break;

	    case "choiceBuff5b":
		var BuffLevel = parseInt(dictSzValue("choiceBuff5b"));
		if (BuffLevel == Buff[AccCurChar].Info[BuffDefIndex].Level) break;
	    	if (!Cast.Know(Buff[AccCurChar].Info[BuffDefIndex].Self[BuffLevel])) {
		    Chat.Screen("WARNING!!! You don't know " + Buff[AccCurChar].Info[BuffDefIndex].Name + " Self " + BuffLevel + ".");
		    if (Buff[AccCurChar].Info[BuffDefIndex].Enabled) {
			Chat.Screen("WARNING!!! Buff disabled!");
			Buff[AccCurChar].Info[BuffDefIndex].Enabled = false;
			skapi.SetControlProperty(szPanelBuffs, "chkBuffDefEnab", "checked", false);
			break;
		    } // endif
		} // endif
		Buff[AccCurChar].Info[BuffDefIndex].SpellObj = skapi.SpellInfoFromSpellid(Buff[AccCurChar].Info[BuffDefIndex].Self[BuffLevel]);
		Buff[AccCurChar].Info[BuffDefIndex].Level = BuffLevel;
		break;

	    case "editBuff5c":
		szExp = dictSzValue("editBuff5c");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffDefIndex].Safety = intExp*1000;
		skapi.SetControlProperty(szPanelBuffs, "text5cv", "text", Buff[AccCurChar].Info[BuffDefIndex].Safety/1000);
		break;

	    case "editBuff5d":
		szExp = dictSzValue("editBuff5d");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		intExp = parseInt(szExp);
		Buff[AccCurChar].Info[BuffDefIndex].Priority = intExp;
		skapi.SetControlProperty(szPanelBuffs, "text5dv", "text", Buff[AccCurChar].Info[BuffDefIndex].Priority);
		break;

	} // end switch
	if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: End, Buff");
    	return;
    } // endif



   // Comp panel
    if (szPanel == szPanelComps) {
   	// Process Control Panel Controls
	switch (szControl) {

	    // G(eneral) tab ******************************************

	    case "btnCompLoad":
	       	Chat.Screen("Loading component configuration info...");
	        var LoadCount = Comp[AccCurChar].Load(AccCurChar);	// Load info from file
	        if (LoadCount>=0) {
		    SetCompControls();	// Now update decal panels with new settings
		    Chat.Screen("Loaded component configuration info from file " + Comp[AccCurChar].FileName);
		} else {
		    Chat.Screen("File " + Comp[AccCurChar].FileName + " not found!");
		} // endif
	        break;

	    case "btnCompSave":
	       	Chat.Screen("Saving component configuration info...");
		Comp[AccCurChar].Save(AccCurChar);
		Chat.Screen("Component configuration info saved to file " + Comp[AccCurChar].FileName);
	        break;

	    case "btnCompReset":
	    	Chat.Screen("Resetting comps...");
		Comp[AccCurChar].Reset();
		Chat.Screen("Component configuration reset to defaults!");
	        break;

	    // C(omponent) tab

	    case "compChoice":
		// Update text for new comp selected
		var CompIndex = parseInt(dictSzValue("compChoice"));
		DisplayComp(CompIndex);
		break;

	    case "chkCompEnabled":
		var CompIndex = parseInt(dictSzValue("compChoice"));
		with (Comp[AccCurChar].Info[CompIndex]) {
		    Enabled = !Enabled;
		    skapi.SetControlProperty(szPanelComps, "chkCompEnabled", "checked", Enabled);
		} // end with
		break;

	    case "editComp1":
		szExp = dictSzValue("editComp1");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var CompIndex = parseInt(dictSzValue("compChoice"));
		intExp = parseInt(szExp);
		with (Comp[AccCurChar].Info[CompIndex]) {
		    WarnLvl = intExp;
		    skapi.SetControlProperty(szPanelComps, "textComp1v", "text", WarnLvl);
		} // end with
		break;

	    case "editComp2":
		szExp = dictSzValue("editComp2");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var CompIndex = parseInt(dictSzValue("compChoice"));
		intExp = parseInt(szExp);
		with (Comp[AccCurChar].Info[CompIndex]) {
		    SplitLvl = intExp;
		    skapi.SetControlProperty(szPanelComps, "textComp2v", "text", SplitLvl);
		} // end with
		break;

	    case "editComp3":
		szExp = dictSzValue("editComp3");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var CompIndex = parseInt(dictSzValue("compChoice"));
		intExp = parseInt(szExp);
		with (Comp[AccCurChar].Info[CompIndex]) {
		    CritLvl = intExp;
		    skapi.SetControlProperty(szPanelComps, "textComp3v", "text", CritLvl);
		} // end with
		break;

	    case "editComp4":
		szExp = dictSzValue("editComp4");
		if (isNaN(szExp) || (szExp == undefined) || (szExp == "")) {
		    Chat.Screen("Invalid number entered");
		    break;
		} // endif
		var CompIndex = parseInt(dictSzValue("compChoice"));
		intExp = parseInt(szExp);
		with (Comp[AccCurChar].Info[CompIndex]) {
		    BuyLvl = intExp;
		    skapi.SetControlProperty(szPanelComps, "textComp4v", "text", BuyLvl);
		} // end with
		break;

	    case "btnCompNext":
		var CompIndex = parseInt(dictSzValue("compChoice"));
		CompIndex++;
		if (CompIndex == Comp[AccCurChar].Count) CompIndex = 0;
		skapi.SetControlProperty(szPanelComps, "compChoice", "selected", CompIndex);
		break;

	    case "btnCompPrev":
		var CompIndex = parseInt(dictSzValue("compChoice"));
		if (CompIndex == 0) CompIndex = Comp[AccCurChar].Count-1; else CompIndex--;
		skapi.SetControlProperty(szPanelComps, "compChoice", "selected", CompIndex);
		break;

	    case "btnCompSetAs":
		var CompIndex = parseInt(dictSzValue("compChoice"));
		var PrevIndex = CompIndex;
		if (PrevIndex == 0) var PrevIndex = Comp[AccCurChar].Count-1; else PrevIndex--;
		Comp[AccCurChar].Info[CompIndex].Enabled  = Comp[AccCurChar].Info[PrevIndex].Enabled;
		Comp[AccCurChar].Info[CompIndex].WarnLvl  = Comp[AccCurChar].Info[PrevIndex].WarnLvl;
		Comp[AccCurChar].Info[CompIndex].SplitLvl = Comp[AccCurChar].Info[PrevIndex].SplitLvl;
		Comp[AccCurChar].Info[CompIndex].CritLvl  = Comp[AccCurChar].Info[PrevIndex].CritLvl;
		DisplayComp(CompIndex);
		break;

	    case "btnCompReset":
		var CompIndex = parseInt(dictSzValue("compChoice"));
		Chat.Screen("Resetting comps...");
		Comp[AccCurChar].Reset();
		DisplayComp(CompIndex);
		Chat.Screen("Component info reset.");
		break;

	    case "btnCompShow":
	        Header = false;
	        for (var x=0; x<Comp[AccCurChar].Count; x++) {
		    if (Comp[AccCurChar].Info[x].Enabled) {
		    	if (!Header) {
		    	    Header = true;
		    	    Chat.Screen(Util.Width("Comp",2.0)  + Util.Width("Warn Lvl",1.0) +
		    	    		Util.Width("Split Lvl",1.0) + Util.Width("Crit Lvl",1.0) + "Buy Lvl");
		    	} // endif
		        with (Comp[AccCurChar].Info[x]) {
			    // Print info
			    Chat.Screen(Util.Width(Name,2.0)  + Util.Width(WarnLvl + " ",1.0) +
		    	    		Util.Width(SplitLvl + " ",1.0) + Util.Width(CritLvl + " ",1.0) + BuyLvl );
			} // end with
		    } // endif
		} // end for
		if (!Header) Chat.Screen("No comps being checked!");
		break;
		
	} // end switch
	if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: End, Comps");
    	return;
    } // endif

    if (debugEvent||debugCmds) Log.OutD("Main_OnControlEvent: End, No switch statement executed");
    return;
} // end function