// PB_SUBS.JS
//
// Author:	Maud-Dib
// Version:	2.06
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
//
// Description:
//
//	This module contains all the subroutines for the WarBot macro
//

function StateChange(NewState,NoCheck) {
    if ((NoCheck == undefined) || (NoCheck == null)) NoCheck = false;

    if (!NoCheck) {
	if ((State == stateHide) || (State == stateDied) || (State == stateInit) ||
	    (State == stateLog) || (State == stateExit)) {
		Log.OutD("StateChange: Current State Locked, Current [" + State + "/" + textState[State] +
			 "] Desired state [" + NewState + "/" + textState[NewState] + "]" );
		return false;
	} // endif
    } // endif

    if (debug||debugState)
	Log.OutD("StateChange: Switching state from [" + State + "/" + textState[State] +
		 "] to [" + NewState + "/" + textState[NewState] + "] NoCheck [" + NoCheck + "]");

    State = NewState;
    return true;
} // end func


function CheckGameState() {
    // Determine game state
    Log.OutD("Main: Determining game state...");
    skapi.WaitEvent(0);
    if (skapi.plig == pligNotRunning) {
    	Log.OutDM("Main", "Game not running, please start AC before starting this macro", ToChat);
    	return false;
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligAtLogin) {
    	Log.OutDM("Main", "Game at Login screen, logging in default char", ToChat);
    	Player.LogOn(accDefChar);
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligInPortal) {
    	Log.OutDM("Main", "Macro started while char in portal space, waiting...", ToChat);
    	var wait_start = new Date();
   	while ((skapi.plig != pligInWorld) && ((new Date()-wait_start) < Player.LogonTimeout))
   		skapi.WaitEvent(10);
   	if (skapi.plig != pligInWorld) {
   	    Log.OutDM("Main", "Timeout waiting to come out of portal space, aborting macro", ToChat);
   	    return false;
   	} // endif
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligInWorld) {
    	// Get current slot # of logged in character for use in auto login functions.
	AccCurChar = Player.ToSlotNum(skapi.szName);
	AccNewChar = AccCurChar;
	if (AccCurChar < 0) {
	    // Couldn't match our name against Acc name list
	    Log.OutD("%Logic error, can't match current char's name against account name");
	    return false;
	} // endif
    	Log.OutDM("Main", "Character " + skapi.szName + " (Slot " + CharSlot + ") in world");
    } else {
    	Log.OutDM("Main", "Character not in world, aborting macro...", ToChat);
    	return false;
    } //endif

    // If we make it here, character is in the world (in-game)
    return true;
} // end func


function PossiblyResetToon() {
    // If players cursor (pointer) is busy, then press the escape key just in case we are
    // stuck doing an action.  During the idle state nothing should be going on.

    var WaitStart = new Date();
    while (skapi.fPointerBusy && ((new Date() - WaitStart) < busyWait)) skapi.WaitEvent(20, wemFullTimeout);

    // If pointer not busy, then return false (no action taken)
    if (!skapi.fPointerBusy) return false;

    // Pointer still busy, try to 'unstick' character   
    Log.OutDM("PossiblyResetToon", "Pointer busy, pressing escape key twice in case we are stuck...");
    skapi.Keys("{ESC}", kmoClick);
    skapi.WaitEvent(300, wemFullTimeout);
    skapi.Keys("{ESC}", kmoClick);
    skapi.WaitEvent(300, wemFullTimeout);
    Log.OutDM("PossiblyResetToon", "Unstuck attempt ended.");
    return true;
} // end func   


function MacroInit() {
    BuffMode	= startBuffMode;
    SummonMode	= startSummonMode;
    
    Player.SetLogoutTimeout(playerLogoutTimeout);
    Player.SetLogonTimeout(playerLoginTimeout);

     // Define log file for cast stats if option enabled
    if (flagLogStats) StatLog = new LogFileObj(fileStatLog, ForAppending, true);

    // Define log file for gifts if option enabled
    if (flagLogGifts) GiftLog = new LogFileObj(fileGiftLog, ForAppending, true);
 
    // Define log file for summon requests if option enabled
    if (flagLogCmds) CmdLog = new LogFileObj(fileCmdLog, ForAppending, true);
    
    // Define log file for local chat if option enabled
    if (flagLogChat||flagLogAlleg) ChatLog = new LogFileObj(fileChatLog, ForAppending, true);	

    // Save current player game state so we can detect changes
    LastPlig = skapi.plig;

    BuildHelp();	// Build help message
} // end func


function ToonInit() {
    if (debug) Log.OutD("ToonInit: Begin");
    
    // Define array that holds information for each character in an account.  Will be filled in later
    Toon = new Array();
    ToonToId = new Array();	// Array indexed by character names that contains chars slot #
    
    // Setup character structures for portal summoning.  We will read information from the
    // character config file to fill in these structures. The array index in the Toon array
    // corresponds to the characters login slot.
    if (debug) Log.OutD("Toon_Init: Initializing toon info structures...");
    for (var x=0; x<maxSlots; x++) {
    	Toon[x] = new ToonObj();
    	if (x<skapi.cchar) {
	    Toon[x].Name = skapi.rgszChar(x);
	} // endif
    } // endif
} // end func


function GemInit() {
    GemCount = portalGems.length;
    for (var x=0; x<GemCount; x++) {
    	GemList[x] = new GemObj(portalGems[x]);
    } // end for
} // end func


function CritSpellsInit() {
    // Define critical spells needed for macro operation
    SpellObj_Heal	= skapi.SpellInfoFromSpellid(spellListHeal[Toon[AccCurChar].SpellHeal]);
    SpellObj_StamMana	= skapi.SpellInfoFromSpellid(spellListStamMana[Toon[AccCurChar].SpellStamMana]);
    SpellObj_Revitalize	= skapi.SpellInfoFromSpellid(spellListRevitalize[Toon[AccCurChar].SpellRevitalize]);
    SpellObj_Strength	= skapi.SpellInfoFromSpellid(spellListStrength[Toon[AccCurChar].SpellStrength]);
} // end func


function CastInit() {
    if (debug) Log.OutD("CastInit: Begin");

    Cast.BuildSpellBook();	// Make sure spellbook is built correctly

    if (debug) Log.OutD("CastInit: End");
} // end function CastInit


function WarEvade(SlideFlag) {
    if (debug||debugEvade) Log.OutD("WarEvade: Begin, SlideFlag [" + SlideFlag + "]");

    // If previous evade happened very recently, then continue in same direction so we don't
    // run into previous war spell
    if ((new Date()-PrevEvadeTime) < avoidHoldDown) {
    	if (debugEvade) {
    	    Log.OutD("WarEvade: Previous evade just occured, continuing in same direction");
    	    if (PrevEvadeKey == cmidSideStepLeft)
    	        Log.OutD("WarEvade: Previous direction was left");
    	    else
    	      	Log.OutD("WarEvade: Previous direction was right");
    	} // endif
    	var KeyPress = PrevEvadeKey;
    } else {
    	// No recent evade, figure out direction that brings us closer to origin
	var OurLoc = skapi.maplocCur;
	var Head2Home = OurLoc.HeadToMaploc(HomeLoc);
	var Diff = Head2Home - OurLoc.head;
	if (Diff<0) Diff = Diff + 360;

	if (debugEvade) {
	    Log.OutD("WarEvade: OurLoc [" + Util.FormatCoords(OurLoc,3) + "] OurHead [" + OurLoc.head +
		     "] Head2Home [" + Head2Home + "]");
	    Log.OutD("WarEvade: HomeLoc [" + Util.FormatCoords(HomeLoc,3) + "] Diff [" + Diff + "]");
	} // endif

	if (Diff>180) {
    	    if (debugEvade) Log.OutD("WarEvade: Sidestepping Left!");
	    var KeyPress = cmidSideStepLeft;	// Sidestep Left
	} else {
    	    if (debugEvade) Log.OutD("WarEvade: Sidestepping Right!");
	    var KeyPress = cmidSideStepRight;	// Sidestep Right
	} // endif

    } // endif

    // If we are slidecasting hold down the sidestep and backwards key, else just sidestep
    if (SlideFlag) {
    	skapi.Keys(KeyPress, kmoDown);
    	skapi.Keys(cmidWalkBackwards, kmoDown);
	skapi.Sleep(avoidTimeSlide);
	skapi.Keys(cmidWalkBackwards, kmoUp);
	skapi.Keys(KeyPress, kmoUp);
    } else {
	skapi.Keys(KeyPress, kmoDown);
	skapi.Sleep(avoidTime);
	skapi.Keys(KeyPress, kmoUp);
    } // endif

    // Record what direction we went in case another war spell comes very quickly
    PrevEvadeTime = new Date();
    PrevEvadeKey  = KeyPress;

    if (debug||debugEvade) Log.OutD("WarEvade: End, Sidestep finished");
    return;
} // end function WarEvade


function UpdateMagicSkills() {
    if (debug) Log.OutD("UpdateMagicSkills: Begin");
    skItem	= skapi.SkinfoFromSkid(skidItemEnchantment);
    skLife	= skapi.SkinfoFromSkid(skidLifeMagic);
    skCre	= skapi.SkinfoFromSkid(skidCreatureEnchantment);

    CharCurItem	= skItem.lvlCur;
    CharCurLife	= skLife.lvlCur;
    CharCurCre	= skCre.lvlCur;

    if (debug) Log.OutD("UpdateMagicSkills: End");
} // end function UpdateMagicSkills


function UpdateVitalVars() {
    if (debug) Log.OutD("UpdateVitalVars: Begin");

    // Get secondary attribute information
    skHealth  = skapi.SkinfoFromVital(vitalHealthMax);
    skStamina = skapi.SkinfoFromVital(vitalStaminaMax);
    skMana    = skapi.SkinfoFromVital(vitalManaMax);

    // Extract current vital maximums (includes debuffs/buffs)
    CharMaxHealth  = skHealth.lvlCur;
    CharMaxStamina = skStamina.lvlCur;
    CharMaxMana	   = skMana.lvlCur;

    // Compute vital recharge thresholds
    ThreshHeal	  = CharMaxHealth  - threshHealOffset; 	  // When health falls below this, then heal self
    ThreshMana	  = CharMaxMana    - threshManaOffset;	  // When mana falls below this, then stam-to-mana
    ThreshStamina = CharMaxStamina - threshStaminaOffset; // When stamina falls below this, then revitalize

    if (debug4) {
	Log.OutD("UpdateVitalVars: CharMaxHealth [" + CharMaxHealth +
				"] CharMaxMana [" + CharMaxMana +
				"] CharMaxStamina [" + CharMaxStamina + "]" );
	Log.OutD("UpdateVitalVars: ThreshHeal [" + ThreshHeal +
    				"] ThreshMana [" + ThreshMana +
    				"] ThreshStamina [" + ThreshStamina + "]" );
    } // endif

    if (debug) Log.OutD("UpdateVitalVars: End");
} // end function


function ToonInfoSet(Name, SpellHealLvl, SpellStamMana, SpellRevit, SpellStrength) {
    var Slot = Player.ToSlotNum(Name);
    if (Slot < 0) return false;
    Toon[Slot].SpellHeal	= SpellHealLvl;
    Toon[Slot].SpellStamMana	= SpellStamMana;
    Toon[Slot].SpellRevitalize	= SpellRevit;
    Toon[Slot].SpellStrength	= SpellStrength;
    return true;
} // end func
	

function ToonPortalSet(Name, PortalNum, EnabledFlag, SummonLvl, Keyword, Dest ) {
    if (Keyword == undefined) Keyword = null;
    if (Dest == undefined)    Dest = null;

    if (Keyword != null) Keyword = Keyword.toUpperCase();
    var Slot = Player.ToSlotNum(Name);
    if (Slot < 0) return false;
    Toon[Slot].plist[PortalNum].Enabled   = EnabledFlag;
    Toon[Slot].plist[PortalNum].SummonLvl = SummonLvl;
    Toon[Slot].plist[PortalNum].Keyword	  = Keyword;
    Toon[Slot].plist[PortalNum].Dest	  = Dest;
} // end func
    

function FindKeyword(Ident,Name) {	
    // Now search for portal keyword in Toon data structure for match
    var PortalIdent= Ident.toUpperCase();
    for (var index=0; index<maxSlots; index++) {
	if (debugEvent2) Log.OutD("FindKeyword: Checking Toon # [" + index + "]");
	for (var pindex=0; pindex<2; pindex++ ) {  // for each port we can summon...
	    if (Toon[index].plist[pindex].Enabled && (Toon[index].plist[pindex].Keyword == PortalIdent)) {
	    	var elapsed = new Date() - Toon[index].plist[pindex].SumLast
	    	if (elapsed < portalMinInt) {
	    	    Respond(Name,"Not enough time elapsed since last cast of this portal. Try " +
				 "again in " + Math.round((portalMinInt-elapsed)/1000) + " seconds." );
	    	    return true;
	    	} // endif
	    	PortalReq = true;
	    	Toon[index].plist[pindex].RqPending = true;
	    	Toon[index].plist[pindex].RqTime    = new Date();
	    	if (debugEvent2) Log.OutD("FindKeyword: Primary portal match");
	    	var OutLine = "Request acknowledged, portal to " + Toon[index].plist[pindex].Keyword;
	    	if (Toon[index].plist[pindex].Dest != null) OutLine += " " + Toon[index].plist[pindex].Dest;
	    	OutLine += " will be summoned";
		Chat.Local(OutLine);
		if (logSummon)
		    Log.OutD("FindKeyword: " + Util.Pad("Portal to: " +Toon[index].plist[pindex].Keyword,35) +
		    			 "Requested by: " + Name)
	    	return true;	
	    } // endif 	       	
	} // endfor       	 		    
    } // end for
    return false;	// no match
} // end function


function FindGem(Ident,Name) {	
    // Now search for town name in data structure portal gems for match.  Abbreviations ok.
    var TownIdent = Ident.toUpperCase();
    var TownLen	  = TownIdent.length;
    var MatchCnt = 0;
    var MatchGem = -1;
    
    for (var x=0; x<GemCount; x++) {
	if (debugEvent2) Log.OutD("FindGem: Checking Gem # [" + x + "]");
	var GemItem = GemList[x].Name.substr(0,TownLen);
	// Chat.Screen("GemItem [" + GemItem + "] TownIdent [" + TownIdent + "]");
	if (TownIdent == GemItem.toUpperCase()) {
	    MatchCnt++;
	    MatchGem = x;
	} // endif
    } // end for
    
    if (MatchCnt == 0) {
	return false;
    } // endif
    
    if (MatchCnt > 1) {
	Respond(Name, "More than one town matches; specify longer abbreviation or use full name");
	return true;
    } // endif

    // If we make it here, than only one match was found.  Make sure we have those gems in inventory
    InvCnt = skapi.CitemOfKindInInv(GemList[MatchGem].Name + portalGemPost);
    if (InvCnt == 0) {
	Respond(Name, "Sorry, out of portal gems to " + GemList[MatchGem].Name);
	return true;
    } // endif

    // See if gem has been used recently
    var elapsed = new Date() - GemList[MatchGem].UseLast;
    if (elapsed < portalMinInt) {
	Respond(Name, "Not enough time elapsed since last use of this gem. Try " +
		      "again in " + Math.round((portalMinInt-elapsed)/1000) + " seconds." );
	return true;
    } // endif

    PortalReq = true;
    GemList[MatchGem].RqPending = true;
    GemList[MatchGem].RqTime    = new Date();
    if (debugEvent2) Log.OutD("FindGem: Gem match [" + MatchGem + "]");
    var OutLine = "Request acknowledged, portal gem to " + GemList[MatchGem].Name + " will be used shortly";
    Chat.Local(OutLine);
    if (logSummon) Log.OutD("FindGem: " + Util.Pad("Gem to   : " + GemList[MatchGem].Name,35) + "Requested by: " + Name);  
    return true;
} // end function


function FindNextActive() {
    var x = AccCurChar+1;
    if (x >= maxSlots) x = 0;

    while (x != AccCurChar) {
	if ((Toon[x].plist[0].Enabled) || (Toon[x].plist[1].Enabled)) return x;
	x++;
	if (x >= maxSlots) x = 0;
    } // end while
    return x;
} // end func


function ListPortals(Name) {
    OutList = "";
    var Found = false;
    for (var x=0; x<maxSlots; x++) {
	for (var y=0; y<2; y++ ) {  // for each port we can summon...
	    if (Toon[x].plist[y].Enabled) {
	    	if (Toon[x].plist[y].Keyword != null) {
		    if (Found) OutList += ", ";
		    Found = true;
		    OutList += Toon[x].plist[y].Keyword;
		    if (Toon[x].plist[y].Dest != null) {
			OutList += " " + Toon[x].plist[y].Dest;
		    } // endif
		} // endif
	    } // endif 	       	
	} // endfor       	 		    
    } // end for

    if (Found) {
	Respond(Name, OutList);
    } else {
	if (Name != "ALLEG") Respond(Name, "My portal list is empty.");
    } // endif
    return true;
} // end func


// Don't user MD_PLAYEROBJ logoff function since we may need to integrate sound alert with logout action
function PB_PlayerLogOff(PlaySound) {
    if (debugLog) Log.OutD("PB_PlayerLogOff: Begin");

    if (debugLog) Log.OutD("PB_PlayerLogOff: Sending logout shortcut...");
    var LogoffStart = new Date();	// Init timer for logoff attempt

    // Logout.  Although we may have already started a logout
    // attempt in the ObjPlayerCre event, we do it again just in case.
    // For a landblock reset this will be the first one
    skapi.Logout();
    
    // Sound alarm if desired
    if (alertHide && PlaySound) alertHideWav.Play();

    // Now make sure we are completely logged out
    if (debugLog) Log.OutD ("PB_PlayerLogOff: Waiting for char to logout...");
    while ((skapi.plig != pligAtLogin) && ((new Date()-LogoffStart) < logoutTimeout))
   	skapi.WaitEvent(40);

    if (debugLog) Log.OutD("PB_PlayerLogOff: AtLogin wait done");

    if (skapi.plig == pligAtLogin) {
	if (debugLog) Log.OutD("PB_PlayerLogOff: End, Toon successfully logged out");
	return true;
    } // endif

    if (debugLog) Log.OutD("PB_PlayerLogOff: End, Timeout" );
    return false;
} // end func


// Function returns True when vitals ok, false when some action was taken
function CheckVitals() {
    if (debug||debugVitals) Log.OutD("CheckVitals: Begin");

    if (flagCheckVitals == false) {
	if (debugVitals) Log.OutD("CheckVitals: CheckVitals disabled, aborting");
	return true;
    } // endif

    if (debugVitals) {
	Log.OutD("CheckVitals: Health  = " + skapi.healthCur  );
 	Log.OutD("CheckVitals: Stamina = " + skapi.staminaCur );
 	Log.OutD("CheckVitals: Mana    = " + skapi.manaCur    );
    } // endif

    // Clear out events which will make sure we have the most recent vitals info
    skapi.WaitEvent(0);

    if ((skapi.healthCur == 0) && (skapi.manaCur == 0) && (skapi.manaCur == 0)) {
    	if (debugVitals) Log.OutD("CheckVitals: All stats 0, aborting check");
    	return true;
    } // endif

    // If mana is low and stamina high enough, then cast stam to mana.  Note when creatures
    // nearby or we are trying to kill a creature we reduce ThreshMana slightly
    if ((skapi.manaCur < ThreshMana) && (skapi.staminaCur > threshStaminaMin)) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Stam2Mana");
	Cast.Cast(SpellObj_StamMana);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Stam2Mana failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Stam2Mana success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif

   // If player stamina below ThreshStamina then stam ourselves
   if (skapi.staminaCur < ThreshStamina) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Revitalize Self");
	Cast.Cast(SpellObj_Revitalize);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Revit failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Revit success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif
    
    // If player health below Heal_Thresh then heal ourselves
    if (skapi.healthCur < ThreshHeal) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Player health [" + skapi.healthCur + "] below heal thresh, casting healself");

        Cast.Cast(SpellObj_Heal);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Heal self failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Heal self success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif

    if (debug||debugVitals) Log.OutD("CheckVitals: End");
    return true;
} // end function


function ReportComps(Who) {
    if (debug) Log.OutD("ReportComps: Begin, Who [" + Who + "]" );
    
    if ((Who == undefined) || (Who == null) || (Who == "")) Who = skapi.szName;

    // For each component we are tracking, report inventory
    var HeadingReq = true;
    for (var x=0; x<Comp[AccCurChar].Info.length; x++) {
    	if (!Comp[AccCurChar].Info[x].Enabled) continue;
    	
	var Name     = Comp[AccCurChar].Info[x].Name;
	var Stock    = skapi.CitemOfKindInInv(Name);
	var WarnLvl  = Comp[AccCurChar].Info[x].WarnLvl;
	var CritLvl  = Comp[AccCurChar].Info[x].CritLvl;
	var SplitLvl = Comp[AccCurChar].Info[x].SplitLvl;
	var BuyLvl   = Comp[AccCurChar].Info[x].BuyLvl;

    	// If we are reporting to a requestor, only show the comps below the warning level
	// if (TellDest && (Stock < WarnLvl)) continue;

	// If first time through, then print heading
    	if (HeadingReq) {
	    HeadingReq = false;
            OutLine = Util.Width("Comp",2.4) + Util.Width("Stock",0.7) + Util.Width("BuyLvl",0.7) +
                       Util.Width("Warn",0.7) + Util.Width("Crit",0.7) + "PeaSplit";
            Respond(Who, OutLine);
	} // endif

	var OutLine = Util.Width(Name,2.4) + Util.Width(Stock+" ",0.7) + Util.Width(BuyLvl+" ",0.7) +
		      Util.Width(WarnLvl+" ",0.7) + Util.Width(CritLvl+" ",0.7) + SplitLvl;

	Respond(Who, OutLine);
    } // endfor
    
    // If no comps found, say so
    if (HeadingReq) Respond(Who, "No comps being checked.");

    if (debug) Log.OutD("ReportComps: End" );
    return;
} // endif


function CheckDanger() {
    if (debug||debugDanger) Log.OutD("CheckDanger: Begin");

      // Create aco collection filter
    var acf = skapi.AcfNew();
    acf.Reset();
    acf.ocm = ocmMonster;
    acf.distMax = Math.max(creWarnDist, creDangerDist);
    var MonList = acf.CoacoGet();
 
    if (debugDanger) Log.OutD("CheckDanger: # of creatures found [" + MonList.Count + "]");

    var OurLoc = skapi.maplocCur;
    CreWarn   = false;
    CreDanger = false;
    
    var x=0;
    while ((x<MonList.Count) && !(CreWarn && CreDanger)) {
	var MonAco  = MonList.Item(x);
	var MonDist = Util.Distance(OurLoc, MonAco.maploc);
    	if (MonDist <= creWarnDist)   CreWarn   = true;
    	if (MonDist <= creDangerDist) CreDanger = true;
    	x++;
    } // end while

    if (CreDangerPrev != CreDanger) {
        if (CreDanger) {
            if (logDanger||debugDanger) Log.OutD("CheckDanger: Creature danger condition detected");
        } else {
            if (logDanger||debugDanger) Log.OutD("CheckDanger: Danger condition cleared");
        } // endif
        CreDangerPrev = CreDanger;
    } // endif
    
    if (CreWarnPrev != CreWarn) {
        if (CreDanger) {
            if (logDanger||debugDanger) Log.OutD("CheckDanger: Creature warning condition detected");
        } else {
            if (logDanger||debugDanger) Log.OutD("CheckDanger: Warning condition cleared");
        } // endif
        CreWarnPrev = CreWarn;
    } // endif

    if (flagCreatureLog && CreDanger) {
	Log.OutDM("CheckDanger", creDangerMsg, ToChat);
	StateChange(stateHide, NoCheck);
	if (creListPlayers) LogPlayers(0.4);
	return;
    } // endif

    // If creature found within warning distance, see if time to spam to support channel
    if (flagCreatureWarn && CreWarn) {
    	if ((new Date()-LastCreWarn) > creWarnInt) {
	    LastCreWarn = new Date();
	    NotifySupport(Util.FormatTime() + " " + creWarnMsg);
	    // Report players in immediate vicinity if enabled
	    if (creListPlayers) LogPlayers(0.4);
	} // endif
    } // endif

    if (debugDanger && CreDanger) Log.OutD("CheckDanger: Danger condition active!");
    
    if (debug||debugDanger) Log.OutD("CheckDanger: End");
} // end function


function LogPlayers(PlayerDist) {
    var PlayerAcf = skapi.AcfNew();
    PlayerAcf.ocm = ocmPlayer;
    PlayerAcf.distMax = PlayerDist;
    var PlayerList = PlayerAcf.CoacoGet();
    OurLoc = skapi.maplocCur;

    Log.OutD("LogPlayers: " + (PlayerList.Count-1) + " players within " + PlayerDist + " map units.");
    for (var x=0; x<PlayerList.Count; x++) {
	var PlayerAco = PlayerList.Item(x);
	// If this Aco is us, then continue with next entry
	if (PlayerAco.szName == skapi.szName) continue;

	// Get player distance.  Exclude players too far from us which means a stale entry in the object table.
	var Dist = Util.Distance(PlayerAco.maploc,OurLoc);

	// Report player distance and heading
	var OutLine = "Name: " + PlayerAco.szName + "  Dist: " + Util.RoundFill(Dist,2);
	if (Dist > 0.01)
	    OutLine = OutLine + "  Heading: " + Util.Round(OurLoc.HeadToMaploc(PlayerAco.maploc),0) +
				"  Coords:  " + Util.FormatCoords(PlayerAco.maploc,2);
	Log.OutD("LogPlayers: " + OutLine);
    } // endfor
} // end func


function BuildGenStats() {
    if (debug) Log.OutD("BuildGenStats: Begin" );

    var CurTime = new Date();
    ElapsedTimeSec  = (CurTime.getTime() - StartTime.getTime())/1000;
    ElapsedTimeMin  = Util.Round(ElapsedTimeSec/60,0);

    if (debug) Log.OutD("BuildGenStats: End" );
    return;
} // endif


// BuildPortalStats - Build an array of portal stats that can be processed by other routines.
// If parameter 'fixed' is set to true, then fixed font spacing will be done instead of proportional
function BuildPortalStats(fixed) {
    if (debug) Log.OutD("BuildPortalStats: Begin");
    var index=0;

    if (fixed==undefined) fixed=false;
    
    if (fixed) {
    	StatList[index++] = Util.Pad("Count",8) + Util.Pad("Last Summon",16) +
    			    Util.Pad("Description",20) + 
    			    Util.Pad("Count",8) + Util.Pad("Last Summon",16) + "Description";	
    } else {
    	StatList[index++] = Util.Width("Count",0.65) + Util.Width("Last Summon",1.5) +
    			    Util.Width("Description",2.0) + 
    			    Util.Width("Count",0.65) + Util.Width("Last Summon",1.5) + "Description";
    } // endif
    
    var lineout = " ";
    var linecount = 0; 
          	    
    for (var x=0; x<skapi.cchar; x++) {
 	for (var y=0; y<2; y++) {
 	    if (Toon[x].plist[y].Enabled) {
 		var SumLast = Toon[x].plist[y].SumLast;
 		var SumCnt  = Number(Toon[x].plist[y].SumCnt);
 		    	
 		if (SumLast == 0)
		    var StringDate = "Never";
 		else
		    var StringDate = Util.FormatDate(SumLast).substr(0,14);
 		    	
 		var Out_SumCnt = SumCnt.toString();
 		if (SumCnt < 100) Out_SumCnt = "0" + Out_SumCnt;
 		if (SumCnt < 10)  Out_SumCnt = "0" + Out_SumCnt;
 
 		var OutDesc = Toon[x].plist[y].Keyword;
 		if (Toon[x].plist[y].Dest != null) OutDesc += " " + Toon[x].plist[y].Dest
 		if (fixed) {
		    lineout = lineout + Util.Pad(Out_SumCnt,8) + Util.Pad(StringDate,16) + Util.Pad(OutDesc,20);
 		} else {
 		    lineout = lineout + Util.Width(Out_SumCnt,0.65) + Util.Width(StringDate,1.5) +
 		    			Util.Width(OutDesc, 2.0);
 		} // endif
 
 		linecount++;
 		if (linecount == 2) {
 		    StatList[index++] = Util.Trim(lineout);
		    lineout = "";
		    linecount = 0;
 		} // endif		        
 	    } // endif
	} // end for
    } // endfor

    if (linecount==1) StatList[index++] = lineout;	
    if (debug) Log.OutD("BuildPortalStats: End");      
} // end function


function ResetStats() {
    StartTime = new Date();
    for (var x=0; x<skapi.cchar; x++) {
 	for (var y=0; y<2; y++) {
	    Toon[x].plist[y].SumLast = 0;
 	    Toon[x].plist[y].SumCnt = 0;
	} // end for
    } // endfor
} // end func


function BuildHelp() {
    BroadHelpCnt = 0;
    BroadHelp[BroadHelpCnt++] = "I am a multi-character portal bot. I accept the following commands:";
    BroadHelp[BroadHelpCnt++] = Util.Width("Summon 'keyword'",2.8) + "Summons portal.  Keywords in CAPS in LIST command.";
    
    if (flagPortalGems)
	BroadHelp[BroadHelpCnt++] = Util.Width("Gem 'town'",2.8)   + "Uses portal gem.  Abbreviations ok";
    
    BroadHelp[BroadHelpCnt++] = Util.Width("List",2.8)		   + "Displays all portal destinations";
    BroadHelp[BroadHelpCnt++] = Util.Width("Loc",2.8)		   + "Reports my location";
    BroadHelp[BroadHelpCnt++] = Util.Width("Stats",2.8)		   + "Show summon statistics";
} // end func


function Authorized(Name) {
    if (debug) Log.OutD("Authorized: Begin" );
    // See if character 'Name' is an authorized person
    for (var index=0; index<authList.length; index++) {
	if (Name == authList[index]) {
	    if (debug) Log.OutD("Authorized: End, Success - On Authorized List" );
	    return true;
	} // endif
    } // endfor

    // See person is in our monarchy
    if (authAllegiance) {
    	// Get chars Aco from his/her name
    	var Aco = skapi.AcoFromSz(Name);
    	if (Aco && (Aco.oidMonarch == skapi.acoChar.oidMonarch)) {
    	    if (debug) Log.OutD("Authorized: End, Success - In monarchy" );
    	    return true;
    	} // endif
    } // endif

    if (debug) Log.OutD("Authorized: End, Failed" );
    return false;	// Not in our allegiance and not on the authorized list
} // end function


function RecordLoc() {
    if (skapi.plig != pligInWorld) return;
    
    var OurMapLoc = skapi.maplocCur;
    if ((OurMapLoc == undefined) || (OurMapLoc == null)) return;

    var DistToLast = Util.Distance(LastRecordedLoc,OurMapLoc);
    if (DistToLast >= locRecordThresh) {
    	Log.OutD("RecordLoc: New Coords [" + Util.FormatCoords(OurMapLoc,2) + "]");
	LastRecordedLoc = OurMapLoc;
    } // endif
} // endif


function SpellCheck(verbose) {
    if ((verbose==undefined) || (verbose==null)) verbose = false;
    if (debug) Log.OutD("SpellCheck: Begin");

    // Assume everything ok
    var ReturnCode = true;
    Restricted = false;

    if (!Cast.Know(SpellObj_StamMana.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Stamina to Mana Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_Heal.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Heal Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_Revitalize.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Revitalize Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (flagReduceBurden && !Cast.Know(SpellObj_Strength.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Strength Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!ReturnCode) {
	Restricted = true;
    	if (verbose) {
	    Log.OutDM("SpellCheck", "WARNING!!! One or more spells necessary for macro operation not defined!!!",
			ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "Macro operating in restricted mode (only some decal panel operations permitted).",
			ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "To fix this condition, select known spells on the P2 tab", ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "on the config panel for the spells listed above.", ToChat, cmcYellow);
	} // endif
    } // endif

    if (debug) Log.OutD("SpellCheck: End");
    return ReturnCode;
} // end func


// This function allows macro to send support requests and/or notification messages
// to friendly chars through @tells and/or @fellow commands
function NotifySupport(request) {
    if (flagChatSup)  Chat.Local(request);
    if (flagAllegSup) Chat.Allegiance(request);
    if (flagFriendSup) {
    	for (var x=0; x<supportChars.length; x++) Chat.Tell(supportChars[x], request);
    } // endif
} // end function
 

function SpellFailText() {
    var Desc = "CC: " + CastCode + " CF: " + CastFail + "/" + castFailText[CastFail] + " CF2: " +
		CastFail2 + " A: " + CastArc + " FC: " + Cast.FailCount + " CFC: " + CastFailCnt;
    return Desc;
} // end func


function PauseMacro() {
    if (debug||debugState) Log.OutD("Main: Pause Begin");
    Log.OutDM("Main", "Macro paused.", ToChat);

    var PauseStart = new Date();

    // Spin until resume command detected
    while (MacroPaused) {
    	skapi.WaitEvent(20);
    	if (flagAutoResume && ((new Date()-PauseStart) > autoResumeInt)) {
    	    Log.OutDM("Main", "Auto resume ending paused state...", ToChat);
    	    MacroPaused = false;
    	    skapi.SetControlProperty(szPanelControl, "btnPause", "Text", "Pause");
    	} // endif
    } // end while

    // If we get here then macro was resumed
    Log.OutDM("Main", "Macro resumed.", ToChat);

    // Pause flag cleared, resume macro.
    // Clear evade war flag so we don't move coming out of Paused state
    EvadeWar = false;

    if (!SummonMode) Chat.Screen("WARNING!!! SummonMode disabled, no portals will be summoned!", cmcYellow);
    if (!BuffMode)   Chat.Screen("WARNING!!! Buffing disabled!", cmcYellow);
    if (debug||debugState) Log.OutD("Main: Pause End");
} // end func


function DisplayComp(CompIndex) {
    with (Comp[AccCurChar].Info[CompIndex]) {
	skapi.SetControlProperty(szPanelComps, "chkCompEnabled", "checked",	Enabled		);
	skapi.SetControlProperty(szPanelComps, "textComp1v",     "text", 	WarnLvl		);
	skapi.SetControlProperty(szPanelComps, "textComp2v",     "text", 	SplitLvl	);
	skapi.SetControlProperty(szPanelComps, "textComp3v",     "text", 	CritLvl		);
	skapi.SetControlProperty(szPanelComps, "textComp4v",     "text", 	BuyLvl		);
    } // end with
} // end func


function DisplayBuffCre(BuffIndex) {
    with (Buff[AccCurChar].Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffCreEnab",    "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffCreRandom",  "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff2b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text2cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text2dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffLife(BuffIndex) {
    with (Buff[AccCurChar].Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff3b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text3cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text3dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffHerm() {
    with (Buff[AccCurChar].Info[BuffHermIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHermEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHermRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff1b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text1cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text1dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffDef() {
    with (Buff[AccCurChar].Info[BuffDefIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffDefEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffDefRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff5b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text5cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text5dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffArmor(BuffIndex) {
    with (Buff[AccCurChar].Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorEnab",   "checked",	 Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorRandom", "checked",	 Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff4b", 	     "selected", 7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text4cv", 	     "text",	 Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text4dv", 	     "text",	 Priority   );

	skapi.SetControlProperty(szPanelBuffs, "chkPaperDoll",	 "checked", (Eqm & eqmPaperDoll)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHead",	 "checked", (Eqm & eqmHead)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHands",	 "checked", (Eqm & eqmHands)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffFeet",	 "checked", (Eqm & eqmFeet)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffChest",	 "checked", (Eqm & eqmChestOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffUpArms",  "checked", (Eqm & eqmUpperArmsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLowArms", "checked", (Eqm & eqmLowerArmsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffAbdomen", "checked", (Eqm & eqmAbdomenOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffUpLegs",	 "checked", (Eqm & eqmUpperLegsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLowLegs", "checked", (Eqm & eqmLowerLegsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffShirt",	 "checked", (Eqm & eqmChestUnder)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffPants",	 "checked", (Eqm & eqmUpperLegsUnder)	);

    } // end width
} // end func

function DisplayConfigPortal(Index) {
    with (Toon[Index]) {
	skapi.SetControlProperty(szPanelConfig, "healSelf", 	 "selected",	7-SpellHeal		);
	skapi.SetControlProperty(szPanelConfig, "stam2mana", 	 "selected",	7-SpellStamMana		);
	skapi.SetControlProperty(szPanelConfig, "revitSelf", 	 "selected",	7-SpellRevitalize	);
	skapi.SetControlProperty(szPanelConfig, "strengthSelf",	 "selected",	7-SpellStrength		);
	
	skapi.SetControlProperty(szPanelConfig, "chkPrimEnabled", "checked",	plist[0].Enabled	);
	skapi.SetControlProperty(szPanelConfig, "chkSecEnabled",  "checked",	plist[1].Enabled	);

	if (plist[0].Keyword != null) {
	    skapi.SetControlProperty(szPanelConfig, "text4av", "text", plist[0].Keyword);
	} else {
	    skapi.SetControlProperty(szPanelConfig, "text4av", "text", " ");
	} // endif
	    
	if (plist[0].Dest != null) {
	    skapi.SetControlProperty(szPanelConfig, "text4bv", "text", plist[0].Dest);
	} else {
	    skapi.SetControlProperty(szPanelConfig, "text4bv", "text", " ");
	} // endif

	if (plist[1].Keyword != null) {
	    skapi.SetControlProperty(szPanelConfig, "text4ev", "text", plist[1].Keyword);
	} else {
	    skapi.SetControlProperty(szPanelConfig, "text4ev", "text", " ");
	} // endif

	if (plist[1].Dest != null) {
	    skapi.SetControlProperty(szPanelConfig, "text4fv", "text", plist[1].Dest);
	} else {
	    skapi.SetControlProperty(szPanelConfig, "text4fv", "text", " ");
	} // endif
    } // end with
} // end func


function GenLoad() {
    if (debug) Log.OutD("GenLoad: Begin");

    // Build the input filename
    GenFileName = fileGenCfg;

    // See if file exists
    if(!Fso.FileExists(GenFileName)) return false;

    // Now read config file
    Util.Include(GenFileName);
    return true;
} // end func

function GenSave() {
    if (debug) BuffLog.OutD("GenSave: Begin");

     // Build the input filename
    GenFileName = fileGenCfg;

    GenFile = Fso.OpenTextFile(GenFileName, ForWriting, true);	// Create config file

    GenFile.WriteLine("// General configuration file created " + Util.FormatDate(0,false) + " by " + skapi.szName);
    GenFile.WriteLine("GEN_BOF = true;");

    // Gen 1 tab variables
    GenFile.WriteLine("flagCreatureLog     = " + flagCreatureLog     + ";");
    GenFile.WriteLine("flagCreatureWarn    = " + flagCreatureWarn    + ";");
    GenFile.WriteLine("flagDeathLog        = " + flagDeathLog        + ";");
    GenFile.WriteLine("flagDamageLog       = " + flagDamageLog       + ";");
    GenFile.WriteLine("flagReduceBurden    = " + flagReduceBurden    + ";");
    GenFile.WriteLine("flagAutoResume      = " + flagAutoResume      + ";");
    GenFile.WriteLine("flagWarAvoid        = " + flagWarAvoid        + ";");
    GenFile.WriteLine("flagSlideCast       = " + flagSlideCast       + ";");
    GenFile.WriteLine("flagRespondWhere    = " + flagRespondWhere    + ";");

    // Portal 1 tab variables
    GenFile.WriteLine("flagCloseOnly       = " + flagCloseOnly       + ";");
    GenFile.WriteLine("flagPortalGems      = " + flagPortalGems      + ";");
    GenFile.WriteLine("flagAutoRotate      = " + flagAutoRotate      + ";");
    GenFile.WriteLine("flagStormAvoid      = " + flagStormAvoid      + ";");
    GenFile.WriteLine("flagAdvertise       = " + flagAdvertise       + ";");
    GenFile.WriteLine("flagOpenMsg         = " + flagOpenMsg         + ";");
    GenFile.WriteLine("flagAckGifts        = " + flagAckGifts        + ";");
    GenFile.WriteLine("flagLogGifts        = " + flagLogGifts        + ";");
    GenFile.WriteLine("flagLogStats        = " + flagLogStats        + ";");
    GenFile.WriteLine("flagLogCmds         = " + flagLogCmds         + ";");
    GenFile.WriteLine("flagAcceptLocal     = " + flagAcceptLocal     + ";");

    // Portal 2 tab variables.  Dump info for each character slot on login screen
    //
    // ToonInfoSet   (CharName, SpellHealLvl, SpellStamMana, SpellRevit, SpellStrength )
    // ToonPortalSet (CharName,      Portal#, EnabledFlag, SummonLvl, Keyword, Dest )
    // ToonPortalSet ("Fred Norris", 1,       true, "TUSKER", "temple" );
    // Primary portal number is 0, secondary is 1 
    for (var x=0; x<skapi.cchar; x++) {
        GenFile.WriteLine("ToonInfoSet( \"" +  Toon[x].Name + "\", " + Toon[x].SpellHeal +
        		   ", " + Toon[x].SpellStamMana + ", " + Toon[x].SpellRevitalize +
        		   ", " + Toon[x].SpellStrength + ");" );
	for (var y=0; y<2; y++) {
	    var OutLine = "ToonPortalSet( \"" + Toon[x].Name + "\", " + y + ", " +
	    		  Toon[x].plist[y].Enabled + ", " + Toon[x].plist[y].SummonLvl;
	    if (Toon[x].plist[y].Keyword != null) {
		OutLine += ", \"" + Toon[x].plist[y].Keyword + "\"";
		if (Toon[x].plist[y].Dest != null) OutLine += ", \"" + Toon[x].plist[y].Dest + "\"";
	    } // endif
	    OutLine += " );";
	    GenFile.WriteLine(OutLine);
	} // end for
    } // end for

    // Info tab variables
    GenFile.WriteLine("flagChatSup         = " + flagChatSup         + ";");
    GenFile.WriteLine("flagAllegSup        = " + flagAllegSup        + ";");
    GenFile.WriteLine("flagFriendSup       = " + flagFriendSup       + ";");
    GenFile.WriteLine("flagLogChat         = " + flagLogChat         + ";");
    GenFile.WriteLine("flagLogAlleg        = " + flagLogAlleg        + ";");
    GenFile.WriteLine("flagLogGeneral      = " + flagLogGeneral      + ";");
    GenFile.WriteLine("flagLogTrade        = " + flagLogTrade        + ";");
    GenFile.WriteLine("flagReportPlayer    = " + flagReportPlayer    + ";");
    GenFile.WriteLine("flagVerbose         = " + flagVerbose         + ";");

    // Startup tab variables
    GenFile.WriteLine("startBuffMode       = " + startBuffMode       + ";");
    GenFile.WriteLine("startSummonMode     = " + startSummonMode     + ";");
    GenFile.WriteLine("creFellow           = " + creFellow           + ";");
 
    // Component tab variables
    GenFile.WriteLine("flagCompCheck       = " + flagCompCheck       + ";");
    GenFile.WriteLine("flagCompLogOff      = " + flagCompLogOff      + ";");
    GenFile.WriteLine("flagCompAssist      = " + flagCompAssist      + ";");
    GenFile.WriteLine("flagCompRetreat     = " + flagCompRetreat     + ";");
    GenFile.WriteLine("flagSplitPeas       = " + flagSplitPeas       + ";");

    // Alert tab variables
    GenFile.WriteLine("alertDeath          = " + alertDeath          + ";");
    GenFile.WriteLine("alertHide           = " + alertHide           + ";");
    GenFile.WriteLine("alertComps          = " + alertComps          + ";");
    GenFile.WriteLine("alertAbort          = " + alertAbort          + ";");

   // Debug tab variables
    GenFile.WriteLine("debug               = " + debug               + ";");
    GenFile.WriteLine("debugEvent          = " + debugEvent          + ";");
    GenFile.WriteLine("debugState          = " + debugState          + ";");
    GenFile.WriteLine("debugIdle           = " + debugIdle           + ";");
    GenFile.WriteLine("debugBuffs          = " + debugBuffs          + ";");
    GenFile.WriteLine("debugCast           = " + debugCast           + ";");
    GenFile.WriteLine("debugSummon         = " + debugSummon         + ";");

    // Write End Of File marker
    GenFile.WriteLine("GEN_EOF = true;");

    // All done, close file and exit
    GenFile.Close();
    return true;
} // end func


function CreateUserConfig(fileName) {
    UserFile = Fso.OpenTextFile(fileName, ForWriting, true);	// Create config file
    UserFile.WriteLine('// User Configuration File');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// If you want to modify a variable in DIB_CONFIG.JS, then copy the variable');
    UserFile.WriteLine('// to this file.  If you modify the variable directly in DIB_CONFIG.JS then');
    UserFile.WriteLine('// you will lose the modification the next upgrade. ');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Some common variable modifications are listed below.');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// These players can give admin commands to us.' );
    UserFile.WriteLine('// authList = new Array();' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// When enabled, characters on this list will receive @tells when important events' );
    UserFile.WriteLine('// happen in the macro, for example low comps, creature detected nearby, and deaths.' );
    UserFile.WriteLine('// Example: supportChars = new Array("Boy Gary","Crackhead Bob");' );
    UserFile.WriteLine('// supportChars = new Array();' );
    UserFile.WriteLine('//');		
    UserFile.WriteLine('// These players on the shitlist.  Will ignore all messages from these players' );
    UserFile.WriteLine('// ignoreList = new Array("Kou","Jhesus","May-hem");' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Dont record chat for these chars (for exmaple bots in vicinity)' );
    UserFile.WriteLine('// chatIgnoreList = new Array();' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Message to broadcast after successful portal open.' );
    UserFile.WriteLine('// broadOpenMsg = "Donations appreciated.";' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Message to broadcast when advertising services.' );
    UserFile.WriteLine('// broadAdverMsg = "Welcome to DibBot " + macroVersion +' );
    UserFile.WriteLine('//		   ", a multi-character portal bot. For instructions @tell me, Help";' );
    UserFile.WriteLine('// broadIntAdver = 6*60*1000;	// Minimum msecs between advertisement requests (def 6 minutes)' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// If you require requestors to be closeby, this is the max distance' );
    UserFile.WriteLine('// they can be away from us.  0.30 is approximately outdoors radar range.' );
    UserFile.WriteLine('// closebyDist = 0.30;	' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// If enabled, how long (ms) to press left key before casting portal.  Used' );
    UserFile.WriteLine('// to rotate character so it doesnt cast a portal on top of previous casts.' );
    UserFile.WriteLine('// autoRotateTime = 250;' );	
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Minimum interval (ms) between portal casts of same portal. Prevents' );
    UserFile.WriteLine('// malicious player from summoning same portal over and over.' );
    UserFile.WriteLine('// portalMinInt = 70*1000;' );
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Warning and danger parameters' );			
    UserFile.WriteLine('// creDangerDist = 0.10;         // If enabled, creature dist where we will log for hideInt ms' );
    UserFile.WriteLine('// creWarnDist   = 0.20;         // Creature distance at which we will sound a warning' );
    UserFile.WriteLine('// creWarnInt    = 3*60*1000;	 // Default spam interval is 3 minutes' );
    UserFile.WriteLine('// hideInt       = 6*60*1000;    // Default hide interval is 6 minutes' );
    UserFile.WriteLine('//');
    UserFile.Close();
    return;
} // end function