// PB_VAR.JS
//
// Author:	Maud-Dib
// Version:	2.06
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
//
// Macro context variables

State = stateInit;
		// Current state we are in. Possible values:
		// stateInit		Init macro context.  Done after every logon
		// stateIdle		Maintenance routines
		// stateBuff		Check buffs
		// stateCheckCompsInit	Get ready to check comps
		// stateCheckComps	Check components
		// stateHide		Person detected in vicinity, log for x minutes
		// stateReset		Logoff/Logon to avoid portal storm
		// stateStormed		We were portal stormed
		// stateDied		We died, exit game and macro
		// stateLog		Log off AC and stop macro
		// stateExit		Stop Macro

SubState = substateNone;
		// Substate depending on primary state

StartTime	= new Date();
Restricted 	= false;	// Set to true when spells necessary to macro operation are not defined.
				// Only certain looting/salvaging operations permitted.
RunFlag		= true; 	// Flag set to true when normal macro state processing should be done.
				// Set to false when macro should be terminated.


// **************************************************************************************************
//
// General  variables

CompSusp	= false;	// Set to true when macro suspended because of low comps
SuspSummonMode	= false;	// Saved SummonMode flag when low comps detected
DetectedPlayer	= "";		// Name of any unknown player detected


// **************************************************************************************************
//
// Summoning context variables and objects

BroadHelp	= new Array();

BroadLastAdver	= 0;			// Time last message advertising service went out

PortalReq	= false;		// Set to true when a portal is requested.
NextChar 	= -1;			// Char # of oldest portal req.  -1 means not defined
NextPortal	= -1;			// Portal (primary or secondary) of oldest. 0=prim, 1=sec, -1=undefined

AccCurChar	= -1;
AccNewChar	= -1;

CastFailures	= 0;			// # of times portal summon failed in a row

LastKeyPress	= new Date();		// Used to prevent inactivity logout.
StatLastDump	= new Date();		// Last time stats were dumped to log file
StatList	= new Array();		// Array to hold portal stats

LastIgnoreTime	= 0;

GemCount	= 0;
GemList		= new Array();

CreFellowTime	= 0;

SwitchChar	= -1;

InitPanels	= false;		// If true, then decal panels will be redisplayed next idle period

// Portal object.  One exists for each primary and secondary portal for each character
function PortalObj() {
    // The following four properties are read in from the genconfig file
    this.Enabled 	= false;	// Is this portal summonable?
    this.Keyword	= null;		// Keyword used to summon portal
    this.Dest		= null;		// If defined, description of destination
    this.SummonLvl	= 1;		// Use summon portal 1 by default
    
    // This set of properties exist for the life of the macro.  They are not saved
    // between runs
    this.RqPending	= false;	// Request for this portal pending?
    this.RqTime		= 0;		// Time request was made
    this.SumCnt		= 0;		// Total # of requests this run
    this.SumLast	= 0;		// Time portal was last summoned successfully
} // end object constructor

function ToonObj() {
    // The following info is set from the genconfig.dat file
    this.Name		= null;		// Name of character
    this.SpellHeal	= 6;		// Spell level for heal self
    this.SpellStamMana	= 6;		// Spell level for stam to mana
    this.SpellRevitalize= 6;		// Spell level for revitalize self
    this.SpellStrength	= 6;		// Spell level for strength self
   
    // List of portals. [0] = Primary, [1] = Secondary
    this.plist		= new Array();
    this.plist[0]	= new PortalObj();
    this.plist[1]	= new PortalObj();
    
    // Last login/logout times.  Not saved permanently.
    this.LastLogin	= 0;		// Last time logged in
    this.LastLogoff	= 0;		// Last time logged out
    this.LoginCnt	= 0;		// # of times logged in
} // end object constructor

function GemObj(Name) {
    // Stores info for each portal gem
    this.Name		= Name;
    this.Stock		= 0;
    this.RqPending	= 0;
    this.RqTime		= 0;
    this.UseCnt		= 0;
    this.UseLast	= 0;
} // end func


// **************************************************************************************************
//
// Player Context variables

VitalsOK	= true;		// Health, mana, stamina are at acceptable levels
Dead		= false;	// Set to true when we die.  Triggers special death processing
StrengthExp	= 0;		// Time to cast next strength spell to reduce burden

CompLastCheck	= 0;		// Time last spell component check was done.  We start out at zero so
				// that comp check happens immediately

CharEndPortal	= false;	// Set when we come out of portal
CharStormWarn	= false;	// Set when portal storm detected
CharOnEnd3D 	= false;	// Set when OnEnd3D event detected
CharSlot	= -1;		// Set at macro startup.  Holds login screen slot # of current character

CharCurItem	= 0;
CharCurLife	= 0;
CharCurCre	= 0;

PauseStart	= 0;		// Last time macro was paused (via logout)
BuffStateStart  = 0;		// Time last buff run started

EvadeWar 	= false;	// Set to true when war spell heading our way and we need to evade it.
PrevEvadeKey	= 0;		// Previous evade direction and time.  Needed so that we don't step
PrevEvadeTime	= 0;		// back into the first war spell when two come close together.
LastSlideTime	= 0;		// Last time we did a slide cast.

DangerPrev	= false;	// Keep track of previous state so we can log danger state changes
Danger		= false;	// Flag set to true when we are in a creature's attack range

LastRecordedLoc	= 0;


// **************************************************************************************************
//
// Decal panel variables

BuffCrePrev	= 0;
BuffCreIndex	= 0;
BuffCreLast	= -1;
BuffLifePrev	= 0;
BuffLifeIndex	= 0;
BuffLifeLast	= -1;
BuffArmorPrev	= 0;
BuffArmorIndex	= 0;
BuffArmorLast	= -1;
BuffHermIndex	= 0;


// **************************************************************************************************
//
// Idle Context

// Variables
LastHourly	= 0;		// Last time we logged hourly stats.
RotateLast	= new Date();	// Last time we turned.  Used for chars that are leeching exp from fellow.
MacroPaused	= false;	// Set by user commands to pause macro operation.
BuffCheckDone	= false;	// Set to true when buff checking done for an idle period

// **************************************************************************************************
//
// Spell Casting Context

// Spell level to use for various critical spells. Filled in later.
SpellLvlStamMana	= 0;
SpellLvlHeal		= 0;
SpellLvlRevitalize	= 0;
SpellLvlStrength	= 0;

SpellObj_StamMana	= null;
SpellObj_Heal		= null;
SpellObj_Revitalize	= null;
SpellObj_Strength	= null;


// **************************************************************************************************
//
// Decal/Command Context

szPanelControl	= "DB Control";
szPanelConfig	= "DB Config";
szPanelBuffs	= "DB Buffs";
szPanelComps	= "DB Comps";

lastWhereto	= 0;


// **************************************************************************************************
//
// File context

Fso		= new ActiveXObject("Scripting.FileSystemObject")
ForWriting	= 2;


// **************************************************************************************************
//
// Creature detection context context
CreDanger	= false;
CreDangerPrev	= false;
CreWarn		= false;
CreWarnPrev	= false;
LastCreWarn	= 0;		// Last time warning message spammed to support channel


// Reset general config variables to program defaults
function GenVarInit() {

    // Gen 1 tab variables   ************************************************************

    flagCreatureLog	= true;		// If true, log when a creature is detected nearby
    flagCreatureWarn	= true;		// If true, warn when creature detected nearby
    flagDeathLog    	= true;		// If true, we will immediately log on death, otherwise macro will exit
    flagDamageLog	= true;		// If true, log when we receive damage
    flagReduceBurden	= false;	// If true, cast strength spell when overburdened   
    flagAutoResume	= false;	// If true, automatically resume from paused state after 'autoResume' ms.
    flagWarAvoid	= true;		// If true, detect and avoid war spells
    flagSlideCast	= true;		// If true, slide cast when war spell heading for us and we are casting a spell
    flagRespondWhere	= true;		// If true, repsond to !whereto commands on local or allegiance chat

    // Portal 1 tab variables
    flagCloseOnly	= true;
    flagPortalGems	= false;
    flagAutoRotate	= true;		// Turn before summoning portal and to prevent inactivity logout
    flagStormAvoid	= true;
    flagAdvertise	= true;
    flagOpenMsg		= true;
    flagAckGifts	= true;		// If true, announce via local chat any gifts made to us
    flagLogGifts	= true;		// If true, log gifts to file
    flagLogStats	= true;		// If true, log summon statics to file
    flagLogCmds		= true;		// If true, log commands issued to file
    flagAcceptLocal	= false;	// If true, look for summon commands in local chat

    // Info/Startup tab variables
    flagChatSup		= true;		// If true, send important macro events to fellow
    flagAllegSup	= true;		// If true, send important macro events to @allegiance channel
    flagFriendSup	= false;	// If true, send important macro events to chars listed in config var 'supportChars'
    flagLogChat		= false;	// If true, log local chat to file
    flagLogAlleg	= false;	// If true, log allegiance chat to file
    flagLogGeneral	= false;	// If true, log general chat to file
    flagLogTrade	= false;	// If true, log trade chat to file
    flagReportPlayer	= false;	// If true, display all characters detected to chat area
    flagVerbose		= false;	// If true, display detailed info when enabling/disabling decal panel options


    // Startup variables   ***************************************************
    startBuffMode	= true;		// Initial value of BuffMode variable
    startSummonMode 	= true;		// Initial value of SummonMode variable
    creFellow		= false;	// If true, create fellow automatically at macro start

    // Component tab variables  ****************************************************
    flagCompCheck	= true;		// If true, check component levels
    flagCompLogOff	= false;	// If true, log when a component is below its critical level,
    					// otherwise macro will go into defense mode and disable automove
    					// when critical level reached for any one component
    flagCompRetreat	= true;		// When true and CompLogOff = false, then when critical level reached goto
					// the current location's retreat point
    flagCompAssist	= false;	// If true, then broadcast comps we need to fellowship
    flagSplitPeas	= true;		// Controls whether we split peas or not


    // Alert tab variables   ********************************************************
    alertDeath		= true;		// When we die, play sound
    alertHide		= true;
    alertComps		= true;
    alertAbort		= true;


    // Debug tab variables ***********************************************************
    debug		= false;	// Will log basic debugging information for non-events
    debugEvent 		= false;	// Will log debug information for events
    debugState		= false;	// Show debug info related to FSM state changes.
    debugIdle		= false;	// Show debug information related to IDLE state processing
    debugBuffs		= false;	// Show debug information related to buffing
    debugCast		= false;	// Show debug information related to casting spells
    debugSummon		= false;	// Show debug information related to summoning portals

} // endif