// PORTALBOT.JS
//
// Author:	Maud-Dib
// Version:	2.06
//
//
// Description:
//
//	This module is the main code for the PortalBot macro. See AAA_README.TXT and
//	CHANGE.TXT files for more details
//
//
// Modification History
//
// Who	  Date	    Version	Description
//
// MD	  06/23/02  1.01	Initial version
//
// MD     07/20/02  1.02	1) Added pre-log warning message is creature within 2X action range.
//
//				2) Added Hide state that will trigger when creature too close to us.
//				   Macro will stay logged for user selectable time.
//
//				3) If creature too close, the current player and monster collection
//				   table will be dumped to see who the asshole is.
//
//				4) All local chat can be optionally logged (yeah! big brother is here!)
//
//				5) SUMMON command no longer necessary to summon portal.  Too many morons
//				   couldn't figure out the simple help.
//
// MD     07/27/02  1.03	1) Added following admin commands:
//				   SWITCH name	Switches to character 'name' on current account
//				   DISABLE	Stop bot from accepting commands
//				   ENABLE	Accept commands from world
//
//				2) Also now log character receiving gift so we can find them easier
//
// MD     08/08/02  1.04	1) If logout/in fails, try up to 3 times.
//				2) Make the turning to the left before casting portal controlled by flag.
//				3) Fixed misnamed variable in rest routine
//
// MD	  10/20/04  2.01	Rewrite using framework from warbot.  Now supports use of starter portal gems
//

function main() {
    // Define log object for debugging information
    Log = new LogObj();
    
    if (!Fso.FolderExists(macroDir)) {
	// Moron didn't unzip files to correct folder.  Hopefully this check will be the end of the
	// 1000 posts I get every release concerning the script error in md_logobj.s line 154 path not found
	Log.OutD("You must unzip files to folder '" + macroDir + "' for the macro to work properly.", true);
	Log.OutD("Read the instructions and try again.", true);
	return;
    } // endif

    // Enable logging to file and write header
    Log.EnableFile(fileRunLog);
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutDM("Main", "Macro starting...");
    Log.OutD("Main: Macro version [" + macroVersion + "]");
    Log.OutD("Main: Skunkworks version [" + skapi.szVersion + "]");
    Log.OutD("Main: RunningOnNT [" + skapi.fRunningOnNT + "]");

    if (skapi.szVersion<"3.3") {
	Log.OutDM("Main", "Skunkworks v3.3 or later required. Aborting...", ToChat);
	return;
    } // endif

    // Make sure our character is in the world
    if (!CheckGameState()) return;

    // Initialize default values in case no config file found
    GenVarInit();
    ToonInit();		// Build structures for each character on account.
    GemInit();		// Build structures for using portal gems

    // Make sure top level configuration directory exists
    if (!Fso.FolderExists(configDir)) {
	// Config directory doesn't exist, then create it
	Fso.CreateFolder(configDir);
	Log.OutDM("Main", "Configuration directory '" + configDir + "' created.", ToChat);
    } // endif
    
    // Make sure configuration directory exists for this account
    if (!Fso.FolderExists(accountDir)) {
	// Character config directory doesn't exist, then create it
	Fso.CreateFolder(accountDir);
	Log.OutDM("Main", "Configuration directory '" + accountDir + "' created for this account.", ToChat);
    } // endif

    Log.OutDM("Main", "Loading general config file...", ToChat);
    if (GenLoad()) {
	Log.OutDM("Main", "General config info loaded from " + GenFileName, ToChat);
    } else {
    	GenSave();
	Log.OutDM("Main", "General config file " + GenFileName +
		  " not found, file created", ToChat, cmcYellow);
    } // endif

    // Process any user defined variables
    var UserConfigFile = fileUserCfg;
    if (Fso.FileExists(UserConfigFile)) {
    	// Read in user config file for this char
    	Log.OutDM("Main", "Loading user defined config from file '" + UserConfigFile + "'...", ToChat);
	Util.Include(UserConfigFile);
	Log.OutDM("Main", "User config loaded.", ToChat);
    } else {
    	CreateUserConfig(UserConfigFile);
    	Log.OutDM("Main", "User config file '" + UserConfigFile +
    		  "' does not exist! Template user config file created.", ToChat, cmcYellow);
    } // endif

    // Setup macro internal variables based on defaults/config info
    MacroInit();

    // Initialize buff database.  For each character we have separate buff structures
    Log.OutDM("Main", "Initializing buff structures...", ToChat);
    AccCharMax = skapi.cchar;
    Buff = new Array();
    for (var x=0; x<AccCharMax; x++) {
	Buff[x] = new BuffObj(false);
	if (debugBuffs) Buff[x].SetDebug(true);
	Buff[x].Init(fileBuffs);
	var LoadCount = Buff[x].Load(x);
	if (LoadCount >= 0)
	    Log.OutDM("Main", LoadCount + " active buffs loaded from file " + Buff[x].FileName, ToChat);
	else
	    Log.OutDM("Main", "No buffs found for char " + skapi.rgszChar(x), ToChat);
    } // end for

    // Load component config files
    Log.OutDM("Main", "Initializing component structures...", ToChat);
    Comp = new Array();
    for (var x=0; x<AccCharMax; x++) {
    	Comp[x] = new CompObj(false);
    	Comp[x].Load(x);
    } // end for

    // Enable reception of Skunkwork events
    Log.OutDM("Main", "Enabling event handlers...");
    fAddHandlers();

    // Define pause delay after we switch from peace to war mode to ensure we don't
    // get 'Too Busy' error messages
    Player.SetModeWait(charModeDelay);	

    Log.OutDM("Main", "Building decal control panel schemas...", ToChat);
    DefineSchemas();

    // While we want macro to run, process the FSM (finite state machine)
    RunFlag = true;
    StateChange(stateInit,NoCheck);

    // While we want macro to run, process the FSM (finite state machine)
    while (RunFlag == true) {
	if (debug||debugState) Log.OutD("Main: FSM top of loop, State [" + State + "/" + textState[State] + "]");

	// If macro paused, then spin until resume flag set
	if (MacroPaused) {
	    PauseMacro();
	} // endif

	// Process all events
	var EventCount = skapi.WaitEvent(0);
	if (EventCount == 0) {
	    // if no events processed, then wait a little bit so FSM doesn't go crazy.
	    // We don't wait if an event was received since something important may have happened.
	    skapi.WaitEvent(10, wemFullTimeout);
	} // endif

	// After logging on, state is set to this
        if (State == stateInit) DoState_Init();

	// Write our current location to the log file if it is has
	// signicantly changed from our previous recorded location.
	RecordLoc();

	// See if any creatures close to us.  This routine will set global variables Danger and/or
	// Critical if creatures detected close to us.
	CheckDanger();

        // Creature detected in vicinity, log for a while
	if (State == stateHide) {
	    Log.OutD("Calling DoState_Hide...");
	    DoState_Hide();
	    continue;
	} // endif
	
	// If flag is set to avoid portal storms and we have received a portal storm warning, then
	// set state to RESET so toon will be logged out/in.
	if (flagStormAvoid && CharStormWarn) {
	    CharStormWarn = false;
	    StateChange(stateReset);
	} // endif

	// Process all events
	var EventCount = skapi.WaitEvent(0);

	if (logPlig && (skapi.plig != LastPlig)) {
	    Log.OutD("Main: Plig Change: Old [" + LastPlig + "] New [" + skapi.plig + "]");
	    LastPlig = skapi.plig;
	} // endif

	if ((State != stateExit) && (skapi.plig == pligNotRunning)) {
    	    Log.OutD("Main: Game not running, stopping macro", true);
    	    StateChange(stateExit,NoCheck);
    	} // endif

	// See if we should evade any war spells
	if (EvadeWar) {
	    if (debugEvade) Log.OutD("FSM: War Spell Detected,  Evading...");
	    WarEvade(false);
	    EvadeWar = false;
	} // endif

	// Since maximum health/stam/mana can change due to buffs/debuffs, set all variables
	// related to vitals every loop
	UpdateVitalVars();

	// Update current skill levels for item/life/creature magics which change often due to buffs/debuffs.
	// Needed so we can determine maximum spell to cast in a particular school.
	UpdateMagicSkills();

        // If we still want to run macro and are not in a paused state, then check vitals
        if ((RunFlag == true) && !MacroPaused && !Restricted) {
            // if vital check detected an issue, then continue with FSM to check will occur quickly
            VitalsOK = CheckVitals();
            if (VitalsOK) {
		if (debugVitals) Log.OutD("FSM: Vitals ok");
	    } else {
	    	if (debugVitals) Log.OutD("FSM: Vitals condition detected");
	    	continue;
	    } // endif

	    // See if we should evade any war spells
	    if (EvadeWar) {
		if (debugEvade) Log.OutD("FSM: War Spell Detected,  Evading...");
		WarEvade(false);
		EvadeWar = false;
	    } // endif
	} else {
	    VitalsOK = true;
	} // end if

	if (debug) Log.OutD("FSM switch: Processing State [" + State + "]" );
	switch (State) {
 	    case stateIdleInit:		// Not attacking or selecting.  Maintenance functions done here
		if (debug) Log.OutD("Calling DoState_IdleInit...");
		DoState_IdleInit();
		break;

	    case stateIdle:		// Main maintenance function decision tree here
		if (debug) Log.OutD("Calling DoState_Idle...");
		DoState_Idle();
		break;

	    case stateSummonInit:	// See if any portals need to be summoned
		if (debug) Log.OutD("Calling DoState_SummonInit...");
		DoState_SummonInit();
		break;

	    case stateSummon:		// Portal needs to be summoned
		if (debug) Log.OutD("Calling DoState_Summon...");
		DoState_Summon();
		break;

	    case stateCheckComps:	// Check components
		if (debug) Log.OutD("Calling DoState_CheckComps...");
		DoState_CheckComps();
		break;

	    case stateReset:		// Need to relog under different character
		if (debug) Log.OutD("Calling DoState_Reset...");
		DoState_Reset();
		break;

	    case stateStormed:		// We have been portal stormed
		Log.OutD("Calling DoState_PortalStormed...");
		DoState_PortalStormed();
		break;

	    case stateDied:		// We died, exit game and macro
		Log.OutD("Calling DoState_Died...");
		DoState_Died();
		break;

	    case stateLog:		// Log off AC
		Log.OutD("Calling DoState_Log...");
		DoState_Log();
		break;

	    case stateExit:		// Stop macro
		if (alertAbort) {
		    for (var x=0; x<alertAbortCount; x++ ) {
			alertAbortWav.Play();
			skapi.Sleep(alertAbortInt);
		    } // end for
		} // endif
		RunFlag = 0;
		break;

	} // End switch
    } // end while

    // Any final logging/cleanup goes here
    Log.OutD("Macro shutting down...", true);

    fRemoveHandlers();
    RemoveInterface();
} // end main